{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# How to Inspect Function and Class Signatures in Python?\n",
    "\n",
    "[![Twitter Handle](https://img.shields.io/badge/Twitter-@gaohongnan-blue?style=social&logo=twitter)](https://twitter.com/gaohongnan)\n",
    "[![LinkedIn Profile](https://img.shields.io/badge/@gaohongnan-blue?style=social&logo=linkedin)](https://linkedin.com/in/gao-hongnan)\n",
    "[![GitHub Profile](https://img.shields.io/badge/GitHub-gao--hongnan-lightgrey?style=social&logo=github)](https://github.com/gao-hongnan)\n",
    "![Tag](https://img.shields.io/badge/Tag-Brain_Dump-red)\n",
    "[![Code](https://img.shields.io/badge/View-Code-blue?style=flat-square&logo=github)](https://github.com/gao-hongnan/omniverse/blob/main/omnivault/utils/inspector/core.py)\n",
    "\n",
    "```{contents}\n",
    ":local:\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "tags": [
     "hide-cell"
    ]
   },
   "outputs": [],
   "source": [
    "import inspect\n",
    "from dataclasses import field, make_dataclass\n",
    "from inspect import Parameter, Signature\n",
    "from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union, _GenericAlias, get_type_hints, overload\n",
    "\n",
    "from pydantic import BaseModel\n",
    "from rich.pretty import pprint\n",
    "from transformers import GPT2LMHeadModel, Trainer, TrainingArguments"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Motivation\n",
    "\n",
    "There are two motivations for why we want to inspect function and class\n",
    "signatures in Python.\n",
    "\n",
    "1.  **Code Introspection in Open Source Projects**: When we are working on open\n",
    "    source projects, we often need to inspect the function and class signatures\n",
    "    of the codebase. This is especially true when we are working on a large\n",
    "    codebase with many functions and classes. In this case, we need to inspect\n",
    "    the function and class signatures to understand how the codebase is\n",
    "    structured and how the functions and classes are used.\n",
    "\n",
    "    Sometimes there are nested abstractions, a child class $\\mathcal{C}_N$ that\n",
    "    inherits from a parent class $\\mathcal{C}_{N-1}$, which in turn inherits\n",
    "    from another parent class $\\mathcal{C}_{N-2}$, and so on. Sometimes the\n",
    "    child class does not immediately show what types of arguments the\n",
    "    constructor can take. In this case, we need to inspect the parent classes to\n",
    "    understand the constructor signature of the child class.\n",
    "\n",
    "2.  **Agent-Based Function Calling**: In agent-based programming, where\n",
    "    automated agents are tasked with executing functions or interacting with\n",
    "    libraries, the risk of 'hallucination'—where an agent attempts to invoke\n",
    "    non-existent methods or improperly structured calls from a external\n",
    "    module/library—is a notable concern. By equipping agents with the capability\n",
    "    to query the actual signatures of libraries' classes or functions, we can\n",
    "    significantly mitigate this risk. This ensures that agents operate based on\n",
    "    accurate, real-time information, thereby improving the reliability and\n",
    "    effectiveness of automated tasks.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Construct Hypothetical Function, Child and Parent Classes"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [],
   "source": [
    "class ParentClass:\n",
    "    \"\"\"This is the parent class.\"\"\"\n",
    "\n",
    "    parent_class_attr = 'a parent class attribute'\n",
    "\n",
    "    def __init__(self, parent_instance_attr: str) -> None:\n",
    "        self.parent_instance_attr = parent_instance_attr\n",
    "\n",
    "    def parent_method(self) -> str:\n",
    "        \"\"\"This is a method in the parent class.\"\"\"\n",
    "        return \"Parent method called\"\n",
    "\n",
    "class ChildClass(ParentClass):\n",
    "    \"\"\"This is a subclass of ParentClass.\"\"\"\n",
    "\n",
    "    # Class attribute\n",
    "    class_attr = 'a class attribute'\n",
    "\n",
    "    # Private and protected attributes\n",
    "    _protected_attr = 'a protected attribute'\n",
    "    __private_attr = 'a private attribute'\n",
    "\n",
    "    def __init__(self, instance_attr: str, parent_instance_attr: str) -> None:\n",
    "        \"\"\"Initialize the instance.\"\"\"\n",
    "        super().__init__(parent_instance_attr)\n",
    "        # Instance attribute\n",
    "        self.instance_attr = instance_attr\n",
    "        self.instance_not_in_constructor_attr = 'an instance attribute not in the constructor'\n",
    "        self._private_instance_attr = 'a private instance attribute'\n",
    "\n",
    "    @property\n",
    "    def read_only_attr(self) -> str:\n",
    "        \"\"\"This is a read-only attribute.\"\"\"\n",
    "        return 'You can read me, but you cannot change me.'\n",
    "\n",
    "    def instance_method(self, arg: str) -> str:\n",
    "        \"\"\"This is an instance method.\"\"\"\n",
    "        return f'Instance method called with argument: {arg}'\n",
    "\n",
    "    @classmethod\n",
    "    def class_method(cls, arg: str) -> str:\n",
    "        \"\"\"This is a class method.\"\"\"\n",
    "        return f'Class method called with argument: {arg}'\n",
    "\n",
    "    @staticmethod\n",
    "    def static_method(arg: str) -> str:\n",
    "        \"\"\"This is a static method.\"\"\"\n",
    "        return f'Static method called with argument: {arg}'\n",
    "\n",
    "    def __str__(self) -> str:\n",
    "        \"\"\"Return a string representation of the instance.\"\"\"\n",
    "        return f'MyClass(instance_attr={self.instance_attr})'\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [],
   "source": [
    "instance_child = ChildClass(instance_attr='an instance attribute', parent_instance_attr='a parent instance attribute')\n",
    "class_child = ChildClass\n",
    "\n",
    "instance_parent = ParentClass(parent_instance_attr='a parent instance attribute')\n",
    "class_parent = ParentClass"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [],
   "source": [
    "def func(a: int, b: str, c: List[int], d: Tuple[str, str], e: Union[int, str], **kwargs: Any) -> str:\n",
    "    return a, b, c, d, e, kwargs"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Inspect All Members"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [],
   "source": [
    "@overload\n",
    "def get_members_of_function_or_method(\n",
    "    func_or_class: Type[object], predicate: Optional[Callable[[Any], bool]] = None\n",
    ") -> List[Tuple[str, Any]]:\n",
    "    ...\n",
    "\n",
    "\n",
    "@overload\n",
    "def get_members_of_function_or_method(\n",
    "    func_or_class: Callable[..., Any], predicate: Optional[Callable[[Any], bool]] = None\n",
    ") -> List[Tuple[str, Any]]:\n",
    "    ...\n",
    "\n",
    "\n",
    "def get_members_of_function_or_method(\n",
    "    func_or_class: Union[Type[object], Callable[..., Any]], predicate: Optional[Callable[[Any], bool]] = None\n",
    ") -> List[Tuple[str, Any]]:\n",
    "    return inspect.getmembers(func_or_class, predicate)\n",
    "\n",
    "def loop_through_members(members: List[Tuple[str, Any]], filter: Optional[str] = None) -> None:\n",
    "    if filter is not None:\n",
    "        members = [member for member in members if filter in member[0]]\n",
    "    for member in members:\n",
    "        name, value = member\n",
    "        print(f'{name}: {value}')"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Our initial goal is to get all signatures and type annotations of a class or function. We can use the `inspect` module to achieve this. The `getmembers` function returns all members of a class or module. We can then filter out the functions and classes and inspect their signatures.\n",
    "\n",
    "However, for our purpose, it may be overkill since it retrieves all members\n",
    "within a module, the scope is very broad, for example, inspecting just the `func`\n",
    "defined will also return all `__globals__`, which may not be what we want."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "__annotations__: {'a': <class 'int'>, 'b': <class 'str'>, 'c': typing.List[int], 'd': typing.Tuple[str, str], 'e': typing.Union[int, str], 'kwargs': typing.Any, 'return': <class 'str'>}\n",
      "__call__: <method-wrapper '__call__' of function object at 0x29bce6310>\n",
      "__class__: <class 'function'>\n",
      "__closure__: None\n",
      "__code__: <code object func at 0x29bcfb2f0, file \"/var/folders/l2/jjqj299126j0gycr9kkkt9xm0000gn/T/ipykernel_3165/2139551385.py\", line 1>\n",
      "__defaults__: None\n",
      "__delattr__: <method-wrapper '__delattr__' of function object at 0x29bce6310>\n",
      "__dict__: {}\n",
      "__dir__: <built-in method __dir__ of function object at 0x29bce6310>\n",
      "__doc__: None\n",
      "__eq__: <method-wrapper '__eq__' of function object at 0x29bce6310>\n",
      "__format__: <built-in method __format__ of function object at 0x29bce6310>\n",
      "__ge__: <method-wrapper '__ge__' of function object at 0x29bce6310>\n",
      "__get__: <method-wrapper '__get__' of function object at 0x29bce6310>\n",
      "__getattribute__: <method-wrapper '__getattribute__' of function object at 0x29bce6310>\n",
      "__globals__: {'__name__': '__main__', '__doc__': 'Automatically created module for IPython interactive environment', '__package__': None, '__loader__': None, '__spec__': None, '__builtin__': <module 'builtins' (built-in)>, '__builtins__': <module 'builtins' (built-in)>, '_ih': ['', 'from transformers import Trainer, GPT2LMHeadModel, TrainingArguments\\nfrom dataclasses import field, make_dataclass\\n\\nimport inspect\\nfrom inspect import Signature, Parameter\\nfrom typing import Any, Callable, Dict, Set, Optional, _GenericAlias, List, Type, Union, get_type_hints, overload, Tuple\\nfrom pydantic import BaseModel\\nfrom rich.pretty import pprint', 'class ParentClass:\\n    \"\"\"This is the parent class.\"\"\"\\n\\n    parent_class_attr = \\'a parent class attribute\\'\\n\\n    def __init__(self, parent_instance_attr: str) -> None:\\n        self.parent_instance_attr = parent_instance_attr\\n\\n    def parent_method(self) -> str:\\n        \"\"\"This is a method in the parent class.\"\"\"\\n        return \"Parent method called\"\\n\\nclass ChildClass(ParentClass):\\n    \"\"\"This is a subclass of ParentClass.\"\"\"\\n\\n    # Class attribute\\n    class_attr = \\'a class attribute\\'\\n\\n    # Private and protected attributes\\n    _protected_attr = \\'a protected attribute\\'\\n    __private_attr = \\'a private attribute\\'\\n\\n    def __init__(self, instance_attr: str, parent_instance_attr: str) -> None:\\n        \"\"\"Initialize the instance.\"\"\"\\n        super().__init__(parent_instance_attr)\\n        # Instance attribute\\n        self.instance_attr = instance_attr\\n        self.instance_not_in_constructor_attr = \\'an instance attribute not in the constructor\\'\\n        self._private_instance_attr = \\'a private instance attribute\\'\\n\\n    @property\\n    def read_only_attr(self) -> str:\\n        \"\"\"This is a read-only attribute.\"\"\"\\n        return \\'You can read me, but you cannot change me.\\'\\n\\n    def instance_method(self, arg: str) -> str:\\n        \"\"\"This is an instance method.\"\"\"\\n        return f\\'Instance method called with argument: {arg}\\'\\n\\n    @classmethod\\n    def class_method(cls, arg: str) -> str:\\n        \"\"\"This is a class method.\"\"\"\\n        return f\\'Class method called with argument: {arg}\\'\\n\\n    @staticmethod\\n    def static_method(arg: str) -> str:\\n        \"\"\"This is a static method.\"\"\"\\n        return f\\'Static method called with argument: {arg}\\'\\n\\n    def __str__(self) -> str:\\n        \"\"\"Return a string representation of the instance.\"\"\"\\n        return f\\'MyClass(instance_attr={self.instance_attr})\\'', \"instance_child = ChildClass(instance_attr='an instance attribute', parent_instance_attr='a parent instance attribute')\\nclass_child = ChildClass\\n\\ninstance_parent = ParentClass(parent_instance_attr='a parent instance attribute')\\nclass_parent = ParentClass\", 'def func(a: int, b: str, c: List[int], d: Tuple[str, str], e: Union[int, str], **kwargs: Any) -> str:\\n    return a, b, c, d, e, kwargs', \"@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Type[object], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\n@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Callable[..., Any], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\ndef get_members_of_function_or_method(\\n    func_or_class: Union[Type[object], Callable[..., Any]], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    return inspect.getmembers(func_or_class, predicate)\\n\\ndef loop_through_members(members: List[Tuple[str, Any]], filter: Optional[str] = None) -> None:\\n    if filter is not None:\\n        members = [member for member in members if filter in member[0]]\\n    for member in members:\\n        name, value = member\\n        print(f'{name}: {value}')\", 'func_all_members = get_members_of_function_or_method(func, predicate=None)\\nloop_through_members(func_all_members)'], '_oh': {}, '_dh': [PosixPath('/Users/gaohn/gaohn/omniverse/omniverse/playbook'), PosixPath('/Users/gaohn/gaohn/omniverse/omniverse/playbook')], 'In': ['', 'from transformers import Trainer, GPT2LMHeadModel, TrainingArguments\\nfrom dataclasses import field, make_dataclass\\n\\nimport inspect\\nfrom inspect import Signature, Parameter\\nfrom typing import Any, Callable, Dict, Set, Optional, _GenericAlias, List, Type, Union, get_type_hints, overload, Tuple\\nfrom pydantic import BaseModel\\nfrom rich.pretty import pprint', 'class ParentClass:\\n    \"\"\"This is the parent class.\"\"\"\\n\\n    parent_class_attr = \\'a parent class attribute\\'\\n\\n    def __init__(self, parent_instance_attr: str) -> None:\\n        self.parent_instance_attr = parent_instance_attr\\n\\n    def parent_method(self) -> str:\\n        \"\"\"This is a method in the parent class.\"\"\"\\n        return \"Parent method called\"\\n\\nclass ChildClass(ParentClass):\\n    \"\"\"This is a subclass of ParentClass.\"\"\"\\n\\n    # Class attribute\\n    class_attr = \\'a class attribute\\'\\n\\n    # Private and protected attributes\\n    _protected_attr = \\'a protected attribute\\'\\n    __private_attr = \\'a private attribute\\'\\n\\n    def __init__(self, instance_attr: str, parent_instance_attr: str) -> None:\\n        \"\"\"Initialize the instance.\"\"\"\\n        super().__init__(parent_instance_attr)\\n        # Instance attribute\\n        self.instance_attr = instance_attr\\n        self.instance_not_in_constructor_attr = \\'an instance attribute not in the constructor\\'\\n        self._private_instance_attr = \\'a private instance attribute\\'\\n\\n    @property\\n    def read_only_attr(self) -> str:\\n        \"\"\"This is a read-only attribute.\"\"\"\\n        return \\'You can read me, but you cannot change me.\\'\\n\\n    def instance_method(self, arg: str) -> str:\\n        \"\"\"This is an instance method.\"\"\"\\n        return f\\'Instance method called with argument: {arg}\\'\\n\\n    @classmethod\\n    def class_method(cls, arg: str) -> str:\\n        \"\"\"This is a class method.\"\"\"\\n        return f\\'Class method called with argument: {arg}\\'\\n\\n    @staticmethod\\n    def static_method(arg: str) -> str:\\n        \"\"\"This is a static method.\"\"\"\\n        return f\\'Static method called with argument: {arg}\\'\\n\\n    def __str__(self) -> str:\\n        \"\"\"Return a string representation of the instance.\"\"\"\\n        return f\\'MyClass(instance_attr={self.instance_attr})\\'', \"instance_child = ChildClass(instance_attr='an instance attribute', parent_instance_attr='a parent instance attribute')\\nclass_child = ChildClass\\n\\ninstance_parent = ParentClass(parent_instance_attr='a parent instance attribute')\\nclass_parent = ParentClass\", 'def func(a: int, b: str, c: List[int], d: Tuple[str, str], e: Union[int, str], **kwargs: Any) -> str:\\n    return a, b, c, d, e, kwargs', \"@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Type[object], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\n@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Callable[..., Any], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\ndef get_members_of_function_or_method(\\n    func_or_class: Union[Type[object], Callable[..., Any]], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    return inspect.getmembers(func_or_class, predicate)\\n\\ndef loop_through_members(members: List[Tuple[str, Any]], filter: Optional[str] = None) -> None:\\n    if filter is not None:\\n        members = [member for member in members if filter in member[0]]\\n    for member in members:\\n        name, value = member\\n        print(f'{name}: {value}')\", 'func_all_members = get_members_of_function_or_method(func, predicate=None)\\nloop_through_members(func_all_members)'], 'Out': {}, 'get_ipython': <bound method InteractiveShell.get_ipython of <ipykernel.zmqshell.ZMQInteractiveShell object at 0x1092e4250>>, 'exit': <IPython.core.autocall.ZMQExitAutocall object at 0x1092e4e20>, 'quit': <IPython.core.autocall.ZMQExitAutocall object at 0x1092e4e20>, 'open': <function open at 0x1075ee3a0>, '_': '', '__': '', '___': '', '__vsc_ipynb_file__': '/Users/gaohn/gaohn/omniverse/omniverse/playbook/how_to_inspect_function_and_class_signatures.ipynb', '_i': \"@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Type[object], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\n@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Callable[..., Any], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\ndef get_members_of_function_or_method(\\n    func_or_class: Union[Type[object], Callable[..., Any]], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    return inspect.getmembers(func_or_class, predicate)\\n\\ndef loop_through_members(members: List[Tuple[str, Any]], filter: Optional[str] = None) -> None:\\n    if filter is not None:\\n        members = [member for member in members if filter in member[0]]\\n    for member in members:\\n        name, value = member\\n        print(f'{name}: {value}')\", '_ii': 'def func(a: int, b: str, c: List[int], d: Tuple[str, str], e: Union[int, str], **kwargs: Any) -> str:\\n    return a, b, c, d, e, kwargs', '_iii': \"instance_child = ChildClass(instance_attr='an instance attribute', parent_instance_attr='a parent instance attribute')\\nclass_child = ChildClass\\n\\ninstance_parent = ParentClass(parent_instance_attr='a parent instance attribute')\\nclass_parent = ParentClass\", '_i1': 'from transformers import Trainer, GPT2LMHeadModel, TrainingArguments\\nfrom dataclasses import field, make_dataclass\\n\\nimport inspect\\nfrom inspect import Signature, Parameter\\nfrom typing import Any, Callable, Dict, Set, Optional, _GenericAlias, List, Type, Union, get_type_hints, overload, Tuple\\nfrom pydantic import BaseModel\\nfrom rich.pretty import pprint', 'Trainer': <class 'transformers.trainer.Trainer'>, 'GPT2LMHeadModel': <class 'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'>, 'TrainingArguments': <class 'transformers.training_args.TrainingArguments'>, 'field': <function field at 0x1073469d0>, 'make_dataclass': <function make_dataclass at 0x1073610d0>, 'inspect': <module 'inspect' from '/opt/homebrew/Caskroom/miniconda/base/envs/omniverse/lib/python3.9/inspect.py'>, 'Signature': <class 'inspect.Signature'>, 'Parameter': <class 'inspect.Parameter'>, 'Any': typing.Any, 'Callable': typing.Callable, 'Dict': typing.Dict, 'Set': typing.Set, 'Optional': typing.Optional, '_GenericAlias': <class 'typing._GenericAlias'>, 'List': typing.List, 'Type': typing.Type, 'Union': typing.Union, 'get_type_hints': <function get_type_hints at 0x1050cbf70>, 'overload': <function overload at 0x1050cf3a0>, 'Tuple': typing.Tuple, 'BaseModel': <class 'pydantic.main.BaseModel'>, 'pprint': <function pprint at 0x294a745e0>, '_i2': 'class ParentClass:\\n    \"\"\"This is the parent class.\"\"\"\\n\\n    parent_class_attr = \\'a parent class attribute\\'\\n\\n    def __init__(self, parent_instance_attr: str) -> None:\\n        self.parent_instance_attr = parent_instance_attr\\n\\n    def parent_method(self) -> str:\\n        \"\"\"This is a method in the parent class.\"\"\"\\n        return \"Parent method called\"\\n\\nclass ChildClass(ParentClass):\\n    \"\"\"This is a subclass of ParentClass.\"\"\"\\n\\n    # Class attribute\\n    class_attr = \\'a class attribute\\'\\n\\n    # Private and protected attributes\\n    _protected_attr = \\'a protected attribute\\'\\n    __private_attr = \\'a private attribute\\'\\n\\n    def __init__(self, instance_attr: str, parent_instance_attr: str) -> None:\\n        \"\"\"Initialize the instance.\"\"\"\\n        super().__init__(parent_instance_attr)\\n        # Instance attribute\\n        self.instance_attr = instance_attr\\n        self.instance_not_in_constructor_attr = \\'an instance attribute not in the constructor\\'\\n        self._private_instance_attr = \\'a private instance attribute\\'\\n\\n    @property\\n    def read_only_attr(self) -> str:\\n        \"\"\"This is a read-only attribute.\"\"\"\\n        return \\'You can read me, but you cannot change me.\\'\\n\\n    def instance_method(self, arg: str) -> str:\\n        \"\"\"This is an instance method.\"\"\"\\n        return f\\'Instance method called with argument: {arg}\\'\\n\\n    @classmethod\\n    def class_method(cls, arg: str) -> str:\\n        \"\"\"This is a class method.\"\"\"\\n        return f\\'Class method called with argument: {arg}\\'\\n\\n    @staticmethod\\n    def static_method(arg: str) -> str:\\n        \"\"\"This is a static method.\"\"\"\\n        return f\\'Static method called with argument: {arg}\\'\\n\\n    def __str__(self) -> str:\\n        \"\"\"Return a string representation of the instance.\"\"\"\\n        return f\\'MyClass(instance_attr={self.instance_attr})\\'', 'ParentClass': <class '__main__.ParentClass'>, 'ChildClass': <class '__main__.ChildClass'>, '_i3': \"instance_child = ChildClass(instance_attr='an instance attribute', parent_instance_attr='a parent instance attribute')\\nclass_child = ChildClass\\n\\ninstance_parent = ParentClass(parent_instance_attr='a parent instance attribute')\\nclass_parent = ParentClass\", 'instance_child': <__main__.ChildClass object at 0x29bce59d0>, 'class_child': <class '__main__.ChildClass'>, 'instance_parent': <__main__.ParentClass object at 0x29bce50a0>, 'class_parent': <class '__main__.ParentClass'>, '_i4': 'def func(a: int, b: str, c: List[int], d: Tuple[str, str], e: Union[int, str], **kwargs: Any) -> str:\\n    return a, b, c, d, e, kwargs', 'func': <function func at 0x29bce6310>, '_i5': \"@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Type[object], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\n@overload\\ndef get_members_of_function_or_method(\\n    func_or_class: Callable[..., Any], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    ...\\n\\n\\ndef get_members_of_function_or_method(\\n    func_or_class: Union[Type[object], Callable[..., Any]], predicate: Optional[Callable[[Any], bool]] = None\\n) -> List[Tuple[str, Any]]:\\n    return inspect.getmembers(func_or_class, predicate)\\n\\ndef loop_through_members(members: List[Tuple[str, Any]], filter: Optional[str] = None) -> None:\\n    if filter is not None:\\n        members = [member for member in members if filter in member[0]]\\n    for member in members:\\n        name, value = member\\n        print(f'{name}: {value}')\", 'get_members_of_function_or_method': <function get_members_of_function_or_method at 0x29bce4160>, 'loop_through_members': <function loop_through_members at 0x29bce61f0>, '_i6': 'func_all_members = get_members_of_function_or_method(func, predicate=None)\\nloop_through_members(func_all_members)', 'func_all_members': [('__annotations__', {'a': <class 'int'>, 'b': <class 'str'>, 'c': typing.List[int], 'd': typing.Tuple[str, str], 'e': typing.Union[int, str], 'kwargs': typing.Any, 'return': <class 'str'>}), ('__call__', <method-wrapper '__call__' of function object at 0x29bce6310>), ('__class__', <class 'function'>), ('__closure__', None), ('__code__', <code object func at 0x29bcfb2f0, file \"/var/folders/l2/jjqj299126j0gycr9kkkt9xm0000gn/T/ipykernel_3165/2139551385.py\", line 1>), ('__defaults__', None), ('__delattr__', <method-wrapper '__delattr__' of function object at 0x29bce6310>), ('__dict__', {}), ('__dir__', <built-in method __dir__ of function object at 0x29bce6310>), ('__doc__', None), ('__eq__', <method-wrapper '__eq__' of function object at 0x29bce6310>), ('__format__', <built-in method __format__ of function object at 0x29bce6310>), ('__ge__', <method-wrapper '__ge__' of function object at 0x29bce6310>), ('__get__', <method-wrapper '__get__' of function object at 0x29bce6310>), ('__getattribute__', <method-wrapper '__getattribute__' of function object at 0x29bce6310>), ('__globals__', {...}), ('__gt__', <method-wrapper '__gt__' of function object at 0x29bce6310>), ('__hash__', <method-wrapper '__hash__' of function object at 0x29bce6310>), ('__init__', <method-wrapper '__init__' of function object at 0x29bce6310>), ('__init_subclass__', <built-in method __init_subclass__ of type object at 0x104d06270>), ('__kwdefaults__', None), ('__le__', <method-wrapper '__le__' of function object at 0x29bce6310>), ('__lt__', <method-wrapper '__lt__' of function object at 0x29bce6310>), ('__module__', '__main__'), ('__name__', 'func'), ('__ne__', <method-wrapper '__ne__' of function object at 0x29bce6310>), ('__new__', <built-in method __new__ of type object at 0x104d06270>), ('__qualname__', 'func'), ('__reduce__', <built-in method __reduce__ of function object at 0x29bce6310>), ('__reduce_ex__', <built-in method __reduce_ex__ of function object at 0x29bce6310>), ('__repr__', <method-wrapper '__repr__' of function object at 0x29bce6310>), ('__setattr__', <method-wrapper '__setattr__' of function object at 0x29bce6310>), ('__sizeof__', <built-in method __sizeof__ of function object at 0x29bce6310>), ('__str__', <method-wrapper '__str__' of function object at 0x29bce6310>), ('__subclasshook__', <built-in method __subclasshook__ of type object at 0x104d06270>)]}\n",
      "__gt__: <method-wrapper '__gt__' of function object at 0x29bce6310>\n",
      "__hash__: <method-wrapper '__hash__' of function object at 0x29bce6310>\n",
      "__init__: <method-wrapper '__init__' of function object at 0x29bce6310>\n",
      "__init_subclass__: <built-in method __init_subclass__ of type object at 0x104d06270>\n",
      "__kwdefaults__: None\n",
      "__le__: <method-wrapper '__le__' of function object at 0x29bce6310>\n",
      "__lt__: <method-wrapper '__lt__' of function object at 0x29bce6310>\n",
      "__module__: __main__\n",
      "__name__: func\n",
      "__ne__: <method-wrapper '__ne__' of function object at 0x29bce6310>\n",
      "__new__: <built-in method __new__ of type object at 0x104d06270>\n",
      "__qualname__: func\n",
      "__reduce__: <built-in method __reduce__ of function object at 0x29bce6310>\n",
      "__reduce_ex__: <built-in method __reduce_ex__ of function object at 0x29bce6310>\n",
      "__repr__: <method-wrapper '__repr__' of function object at 0x29bce6310>\n",
      "__setattr__: <method-wrapper '__setattr__' of function object at 0x29bce6310>\n",
      "__sizeof__: <built-in method __sizeof__ of function object at 0x29bce6310>\n",
      "__str__: <method-wrapper '__str__' of function object at 0x29bce6310>\n",
      "__subclasshook__: <built-in method __subclasshook__ of type object at 0x104d06270>\n"
     ]
    }
   ],
   "source": [
    "func_all_members = get_members_of_function_or_method(func, predicate=None)\n",
    "loop_through_members(func_all_members)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "And to get the signature, we can just filter `'__annotations__'`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "__annotations__: {'a': <class 'int'>, 'b': <class 'str'>, 'c': typing.List[int], 'd': typing.Tuple[str, str], 'e': typing.Union[int, str], 'kwargs': typing.Any, 'return': <class 'str'>}\n"
     ]
    }
   ],
   "source": [
    "loop_through_members(func_all_members, filter='__annotations__')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "_ChildClass__private_attr: a private attribute\n",
      "__class__: <class 'type'>\n",
      "__delattr__: <slot wrapper '__delattr__' of 'object' objects>\n",
      "__dict__: {'__module__': '__main__', '__doc__': 'This is a subclass of ParentClass.', 'class_attr': 'a class attribute', '_protected_attr': 'a protected attribute', '_ChildClass__private_attr': 'a private attribute', '__init__': <function ChildClass.__init__ at 0x29bce64c0>, 'read_only_attr': <property object at 0x29bcec040>, 'instance_method': <function ChildClass.instance_method at 0x29bce6700>, 'class_method': <classmethod object at 0x29bcd5580>, 'static_method': <staticmethod object at 0x29bcd5340>, '__str__': <function ChildClass.__str__ at 0x29bce68b0>}\n",
      "__dir__: <method '__dir__' of 'object' objects>\n",
      "__doc__: This is a subclass of ParentClass.\n",
      "__eq__: <slot wrapper '__eq__' of 'object' objects>\n",
      "__format__: <method '__format__' of 'object' objects>\n",
      "__ge__: <slot wrapper '__ge__' of 'object' objects>\n",
      "__getattribute__: <slot wrapper '__getattribute__' of 'object' objects>\n",
      "__gt__: <slot wrapper '__gt__' of 'object' objects>\n",
      "__hash__: <slot wrapper '__hash__' of 'object' objects>\n",
      "__init__: <function ChildClass.__init__ at 0x29bce64c0>\n",
      "__init_subclass__: <built-in method __init_subclass__ of type object at 0x29b7a5190>\n",
      "__le__: <slot wrapper '__le__' of 'object' objects>\n",
      "__lt__: <slot wrapper '__lt__' of 'object' objects>\n",
      "__module__: __main__\n",
      "__ne__: <slot wrapper '__ne__' of 'object' objects>\n",
      "__new__: <built-in method __new__ of type object at 0x104d1ab00>\n",
      "__reduce__: <method '__reduce__' of 'object' objects>\n",
      "__reduce_ex__: <method '__reduce_ex__' of 'object' objects>\n",
      "__repr__: <slot wrapper '__repr__' of 'object' objects>\n",
      "__setattr__: <slot wrapper '__setattr__' of 'object' objects>\n",
      "__sizeof__: <method '__sizeof__' of 'object' objects>\n",
      "__str__: <function ChildClass.__str__ at 0x29bce68b0>\n",
      "__subclasshook__: <built-in method __subclasshook__ of type object at 0x29b7a5190>\n",
      "__weakref__: <attribute '__weakref__' of 'ParentClass' objects>\n",
      "_protected_attr: a protected attribute\n",
      "class_attr: a class attribute\n",
      "class_method: <bound method ChildClass.class_method of <class '__main__.ChildClass'>>\n",
      "instance_method: <function ChildClass.instance_method at 0x29bce6700>\n",
      "parent_class_attr: a parent class attribute\n",
      "parent_method: <function ParentClass.parent_method at 0x29bce65e0>\n",
      "read_only_attr: <property object at 0x29bcec040>\n",
      "static_method: <function ChildClass.static_method at 0x29bce6820>\n"
     ]
    }
   ],
   "source": [
    "class_child_all_members = get_members_of_function_or_method(class_child, predicate=None)\n",
    "loop_through_members(class_child_all_members)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "_ChildClass__private_attr: a private attribute\n",
      "__class__: <class '__main__.ChildClass'>\n",
      "__delattr__: <method-wrapper '__delattr__' of ChildClass object at 0x29bce59d0>\n",
      "__dict__: {'parent_instance_attr': 'a parent instance attribute', 'instance_attr': 'an instance attribute', 'instance_not_in_constructor_attr': 'an instance attribute not in the constructor', '_private_instance_attr': 'a private instance attribute'}\n",
      "__dir__: <built-in method __dir__ of ChildClass object at 0x29bce59d0>\n",
      "__doc__: This is a subclass of ParentClass.\n",
      "__eq__: <method-wrapper '__eq__' of ChildClass object at 0x29bce59d0>\n",
      "__format__: <built-in method __format__ of ChildClass object at 0x29bce59d0>\n",
      "__ge__: <method-wrapper '__ge__' of ChildClass object at 0x29bce59d0>\n",
      "__getattribute__: <method-wrapper '__getattribute__' of ChildClass object at 0x29bce59d0>\n",
      "__gt__: <method-wrapper '__gt__' of ChildClass object at 0x29bce59d0>\n",
      "__hash__: <method-wrapper '__hash__' of ChildClass object at 0x29bce59d0>\n",
      "__init__: <bound method ChildClass.__init__ of <__main__.ChildClass object at 0x29bce59d0>>\n",
      "__init_subclass__: <built-in method __init_subclass__ of type object at 0x29b7a5190>\n",
      "__le__: <method-wrapper '__le__' of ChildClass object at 0x29bce59d0>\n",
      "__lt__: <method-wrapper '__lt__' of ChildClass object at 0x29bce59d0>\n",
      "__module__: __main__\n",
      "__ne__: <method-wrapper '__ne__' of ChildClass object at 0x29bce59d0>\n",
      "__new__: <built-in method __new__ of type object at 0x104d1ab00>\n",
      "__reduce__: <built-in method __reduce__ of ChildClass object at 0x29bce59d0>\n",
      "__reduce_ex__: <built-in method __reduce_ex__ of ChildClass object at 0x29bce59d0>\n",
      "__repr__: <method-wrapper '__repr__' of ChildClass object at 0x29bce59d0>\n",
      "__setattr__: <method-wrapper '__setattr__' of ChildClass object at 0x29bce59d0>\n",
      "__sizeof__: <built-in method __sizeof__ of ChildClass object at 0x29bce59d0>\n",
      "__str__: <bound method ChildClass.__str__ of <__main__.ChildClass object at 0x29bce59d0>>\n",
      "__subclasshook__: <built-in method __subclasshook__ of type object at 0x29b7a5190>\n",
      "__weakref__: None\n",
      "_private_instance_attr: a private instance attribute\n",
      "_protected_attr: a protected attribute\n",
      "class_attr: a class attribute\n",
      "class_method: <bound method ChildClass.class_method of <class '__main__.ChildClass'>>\n",
      "instance_attr: an instance attribute\n",
      "instance_method: <bound method ChildClass.instance_method of <__main__.ChildClass object at 0x29bce59d0>>\n",
      "instance_not_in_constructor_attr: an instance attribute not in the constructor\n",
      "parent_class_attr: a parent class attribute\n",
      "parent_instance_attr: a parent instance attribute\n",
      "parent_method: <bound method ParentClass.parent_method of <__main__.ChildClass object at 0x29bce59d0>>\n",
      "read_only_attr: You can read me, but you cannot change me.\n",
      "static_method: <function ChildClass.static_method at 0x29bce6820>\n"
     ]
    }
   ],
   "source": [
    "instance_child_all_members = get_members_of_function_or_method(instance_child, predicate=None)\n",
    "loop_through_members(instance_child_all_members)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "__class__: <class 'type'>\n",
      "__delattr__: <slot wrapper '__delattr__' of 'object' objects>\n",
      "__dict__: {'__module__': 'transformers.trainer', '__doc__': \"\\n    Trainer is a simple but feature-complete training and eval loop for PyTorch, optimized for 🤗 Transformers.\\n\\n    Args:\\n        model ([`PreTrainedModel`] or `torch.nn.Module`, *optional*):\\n            The model to train, evaluate or use for predictions. If not provided, a `model_init` must be passed.\\n\\n            <Tip>\\n\\n            [`Trainer`] is optimized to work with the [`PreTrainedModel`] provided by the library. You can still use\\n            your own models defined as `torch.nn.Module` as long as they work the same way as the 🤗 Transformers\\n            models.\\n\\n            </Tip>\\n\\n        args ([`TrainingArguments`], *optional*):\\n            The arguments to tweak for training. Will default to a basic instance of [`TrainingArguments`] with the\\n            `output_dir` set to a directory named *tmp_trainer* in the current directory if not provided.\\n        data_collator (`DataCollator`, *optional*):\\n            The function to use to form a batch from a list of elements of `train_dataset` or `eval_dataset`. Will\\n            default to [`default_data_collator`] if no `tokenizer` is provided, an instance of\\n            [`DataCollatorWithPadding`] otherwise.\\n        train_dataset (`torch.utils.data.Dataset` or `torch.utils.data.IterableDataset`, *optional*):\\n            The dataset to use for training. If it is a [`~datasets.Dataset`], columns not accepted by the\\n            `model.forward()` method are automatically removed.\\n\\n            Note that if it's a `torch.utils.data.IterableDataset` with some randomization and you are training in a\\n            distributed fashion, your iterable dataset should either use a internal attribute `generator` that is a\\n            `torch.Generator` for the randomization that must be identical on all processes (and the Trainer will\\n            manually set the seed of this `generator` at each epoch) or have a `set_epoch()` method that internally\\n            sets the seed of the RNGs used.\\n        eval_dataset (Union[`torch.utils.data.Dataset`, Dict[str, `torch.utils.data.Dataset`]), *optional*):\\n             The dataset to use for evaluation. If it is a [`~datasets.Dataset`], columns not accepted by the\\n             `model.forward()` method are automatically removed. If it is a dictionary, it will evaluate on each\\n             dataset prepending the dictionary key to the metric name.\\n        tokenizer ([`PreTrainedTokenizerBase`], *optional*):\\n            The tokenizer used to preprocess the data. If provided, will be used to automatically pad the inputs to the\\n            maximum length when batching inputs, and it will be saved along the model to make it easier to rerun an\\n            interrupted training or reuse the fine-tuned model.\\n        model_init (`Callable[[], PreTrainedModel]`, *optional*):\\n            A function that instantiates the model to be used. If provided, each call to [`~Trainer.train`] will start\\n            from a new instance of the model as given by this function.\\n\\n            The function may have zero argument, or a single one containing the optuna/Ray Tune/SigOpt trial object, to\\n            be able to choose different architectures according to hyper parameters (such as layer count, sizes of\\n            inner layers, dropout probabilities etc).\\n        compute_metrics (`Callable[[EvalPrediction], Dict]`, *optional*):\\n            The function that will be used to compute metrics at evaluation. Must take a [`EvalPrediction`] and return\\n            a dictionary string to metric values.\\n        callbacks (List of [`TrainerCallback`], *optional*):\\n            A list of callbacks to customize the training loop. Will add those to the list of default callbacks\\n            detailed in [here](callback).\\n\\n            If you want to remove one of the default callbacks used, use the [`Trainer.remove_callback`] method.\\n        optimizers (`Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LambdaLR]`, *optional*, defaults to `(None, None)`):\\n            A tuple containing the optimizer and the scheduler to use. Will default to an instance of [`AdamW`] on your\\n            model and a scheduler given by [`get_linear_schedule_with_warmup`] controlled by `args`.\\n        preprocess_logits_for_metrics (`Callable[[torch.Tensor, torch.Tensor], torch.Tensor]`, *optional*):\\n            A function that preprocess the logits right before caching them at each evaluation step. Must take two\\n            tensors, the logits and the labels, and return the logits once processed as desired. The modifications made\\n            by this function will be reflected in the predictions received by `compute_metrics`.\\n\\n            Note that the labels (second parameter) will be `None` if the dataset does not have them.\\n\\n    Important attributes:\\n\\n        - **model** -- Always points to the core model. If using a transformers model, it will be a [`PreTrainedModel`]\\n          subclass.\\n        - **model_wrapped** -- Always points to the most external model in case one or more other modules wrap the\\n          original model. This is the model that should be used for the forward pass. For example, under `DeepSpeed`,\\n          the inner model is wrapped in `DeepSpeed` and then again in `torch.nn.DistributedDataParallel`. If the inner\\n          model hasn't been wrapped, then `self.model_wrapped` is the same as `self.model`.\\n        - **is_model_parallel** -- Whether or not a model has been switched to a model parallel mode (different from\\n          data parallelism, this means some of the model layers are split on different GPUs).\\n        - **place_model_on_device** -- Whether or not to automatically place the model on the device - it will be set\\n          to `False` if model parallel or deepspeed is used, or if the default\\n          `TrainingArguments.place_model_on_device` is overridden to return `False` .\\n        - **is_in_train** -- Whether or not a model is currently running `train` (e.g. when `evaluate` is called while\\n          in `train`)\\n\\n    \", '_get_learning_rate': <function _get_learning_rate at 0x14c9e6940>, 'log_metrics': <function log_metrics at 0x14c9e7280>, 'metrics_format': <function metrics_format at 0x14c9e71f0>, 'save_metrics': <function save_metrics at 0x14c9e7310>, 'save_state': <function save_state at 0x14c9e73a0>, '__init__': <function Trainer.__init__ at 0x29bcacdc0>, '_activate_neftune': <function Trainer._activate_neftune at 0x29bcace50>, '_deactivate_neftune': <function Trainer._deactivate_neftune at 0x29bcacee0>, 'add_callback': <function Trainer.add_callback at 0x29bcacf70>, 'pop_callback': <function Trainer.pop_callback at 0x29bcae040>, 'remove_callback': <function Trainer.remove_callback at 0x29bcae0d0>, '_move_model_to_device': <function Trainer._move_model_to_device at 0x29bcae160>, '_set_signature_columns_if_needed': <function Trainer._set_signature_columns_if_needed at 0x29bcae1f0>, '_remove_unused_columns': <function Trainer._remove_unused_columns at 0x29bcae280>, '_get_collator_with_removed_columns': <function Trainer._get_collator_with_removed_columns at 0x29bcae310>, '_get_train_sampler': <function Trainer._get_train_sampler at 0x29bcae3a0>, 'get_train_dataloader': <function Trainer.get_train_dataloader at 0x29bcae430>, '_get_eval_sampler': <function Trainer._get_eval_sampler at 0x29bcae4c0>, 'get_eval_dataloader': <function Trainer.get_eval_dataloader at 0x29bcae550>, 'get_test_dataloader': <function Trainer.get_test_dataloader at 0x29bcae5e0>, 'create_optimizer_and_scheduler': <function Trainer.create_optimizer_and_scheduler at 0x29bcae670>, 'get_decay_parameter_names': <function Trainer.get_decay_parameter_names at 0x29bcae700>, 'create_optimizer': <function Trainer.create_optimizer at 0x29bcae790>, 'get_optimizer_cls_and_kwargs': <staticmethod object at 0x29bbf71c0>, 'create_scheduler': <function Trainer.create_scheduler at 0x29bcae8b0>, 'num_examples': <function Trainer.num_examples at 0x29bcae940>, 'num_tokens': <function Trainer.num_tokens at 0x29bcae9d0>, '_hp_search_setup': <function Trainer._hp_search_setup at 0x29bcaea60>, '_report_to_hp_search': <function Trainer._report_to_hp_search at 0x29bcaeaf0>, '_tune_save_checkpoint': <function Trainer._tune_save_checkpoint at 0x29bcaeb80>, 'call_model_init': <function Trainer.call_model_init at 0x29bcaec10>, 'torch_jit_model_eval': <function Trainer.torch_jit_model_eval at 0x29bcaeca0>, 'ipex_optimize_model': <function Trainer.ipex_optimize_model at 0x29bcaed30>, '_wrap_model': <function Trainer._wrap_model at 0x29bcaedc0>, 'train': <function Trainer.train at 0x29bcaee50>, '_inner_training_loop': <function Trainer._inner_training_loop at 0x29bcaeee0>, '_get_output_dir': <function Trainer._get_output_dir at 0x29bcaef70>, '_load_from_checkpoint': <function Trainer._load_from_checkpoint at 0x29bcb0040>, '_load_best_model': <function Trainer._load_best_model at 0x29bcb00d0>, '_issue_warnings_after_load': <function Trainer._issue_warnings_after_load at 0x29bcb0160>, '_maybe_log_save_evaluate': <function Trainer._maybe_log_save_evaluate at 0x29bcb01f0>, '_load_rng_state': <function Trainer._load_rng_state at 0x29bcb0280>, '_save_checkpoint': <function Trainer._save_checkpoint at 0x29bcb0310>, '_save_rng_state': <function Trainer._save_rng_state at 0x29bcb03a0>, '_save_optimizer_and_scheduler': <function Trainer._save_optimizer_and_scheduler at 0x29bcb0430>, '_load_optimizer_and_scheduler': <function Trainer._load_optimizer_and_scheduler at 0x29bcb04c0>, 'hyperparameter_search': <function Trainer.hyperparameter_search at 0x29bcb0550>, 'log': <function Trainer.log at 0x29bcb05e0>, '_prepare_input': <function Trainer._prepare_input at 0x29bcb0670>, '_prepare_inputs': <function Trainer._prepare_inputs at 0x29bcb0700>, 'compute_loss_context_manager': <function Trainer.compute_loss_context_manager at 0x29bcb0790>, 'autocast_smart_context_manager': <function Trainer.autocast_smart_context_manager at 0x29bcb0820>, 'training_step': <function Trainer.training_step at 0x29bcb08b0>, 'compute_loss': <function Trainer.compute_loss at 0x29bcb0940>, 'is_local_process_zero': <function Trainer.is_local_process_zero at 0x29bcb09d0>, 'is_world_process_zero': <function Trainer.is_world_process_zero at 0x29bcb0a60>, 'save_model': <function Trainer.save_model at 0x29bcb0af0>, '_save_tpu': <function Trainer._save_tpu at 0x29bcb0b80>, '_save': <function Trainer._save at 0x29bcb0c10>, 'store_flos': <function Trainer.store_flos at 0x29bcb0ca0>, '_sorted_checkpoints': <function Trainer._sorted_checkpoints at 0x29bcb0d30>, '_rotate_checkpoints': <function Trainer._rotate_checkpoints at 0x29bcb0dc0>, 'evaluate': <function Trainer.evaluate at 0x29bcb0e50>, 'predict': <function Trainer.predict at 0x29bcb0ee0>, 'evaluation_loop': <function Trainer.evaluation_loop at 0x29bcb0f70>, '_nested_gather': <function Trainer._nested_gather at 0x29bcb3040>, 'prediction_step': <function Trainer.prediction_step at 0x29bcb30d0>, 'floating_point_ops': <function Trainer.floating_point_ops at 0x29bcb3160>, 'init_hf_repo': <function Trainer.init_hf_repo at 0x29bcb31f0>, 'create_model_card': <function Trainer.create_model_card at 0x29bcb3280>, '_push_from_checkpoint': <function Trainer._push_from_checkpoint at 0x29bcb3310>, '_finish_current_push': <function Trainer._finish_current_push at 0x29bcb33a0>, 'push_to_hub': <function Trainer.push_to_hub at 0x29bcb3430>, 'prediction_loop': <function Trainer.prediction_loop at 0x29bcb34c0>, '_gather_and_numpify': <function Trainer._gather_and_numpify at 0x29bcb3550>, '_add_sm_patterns_to_gitignore': <function Trainer._add_sm_patterns_to_gitignore at 0x29bcb35e0>, 'create_accelerator_and_postprocess': <function Trainer.create_accelerator_and_postprocess at 0x29bcb3670>, 'propagate_args_to_deepspeed': <function Trainer.propagate_args_to_deepspeed at 0x29bcb3700>, '__dict__': <attribute '__dict__' of 'Trainer' objects>, '__weakref__': <attribute '__weakref__' of 'Trainer' objects>}\n",
      "__dir__: <method '__dir__' of 'object' objects>\n",
      "__doc__: \n",
      "    Trainer is a simple but feature-complete training and eval loop for PyTorch, optimized for 🤗 Transformers.\n",
      "\n",
      "    Args:\n",
      "        model ([`PreTrainedModel`] or `torch.nn.Module`, *optional*):\n",
      "            The model to train, evaluate or use for predictions. If not provided, a `model_init` must be passed.\n",
      "\n",
      "            <Tip>\n",
      "\n",
      "            [`Trainer`] is optimized to work with the [`PreTrainedModel`] provided by the library. You can still use\n",
      "            your own models defined as `torch.nn.Module` as long as they work the same way as the 🤗 Transformers\n",
      "            models.\n",
      "\n",
      "            </Tip>\n",
      "\n",
      "        args ([`TrainingArguments`], *optional*):\n",
      "            The arguments to tweak for training. Will default to a basic instance of [`TrainingArguments`] with the\n",
      "            `output_dir` set to a directory named *tmp_trainer* in the current directory if not provided.\n",
      "        data_collator (`DataCollator`, *optional*):\n",
      "            The function to use to form a batch from a list of elements of `train_dataset` or `eval_dataset`. Will\n",
      "            default to [`default_data_collator`] if no `tokenizer` is provided, an instance of\n",
      "            [`DataCollatorWithPadding`] otherwise.\n",
      "        train_dataset (`torch.utils.data.Dataset` or `torch.utils.data.IterableDataset`, *optional*):\n",
      "            The dataset to use for training. If it is a [`~datasets.Dataset`], columns not accepted by the\n",
      "            `model.forward()` method are automatically removed.\n",
      "\n",
      "            Note that if it's a `torch.utils.data.IterableDataset` with some randomization and you are training in a\n",
      "            distributed fashion, your iterable dataset should either use a internal attribute `generator` that is a\n",
      "            `torch.Generator` for the randomization that must be identical on all processes (and the Trainer will\n",
      "            manually set the seed of this `generator` at each epoch) or have a `set_epoch()` method that internally\n",
      "            sets the seed of the RNGs used.\n",
      "        eval_dataset (Union[`torch.utils.data.Dataset`, Dict[str, `torch.utils.data.Dataset`]), *optional*):\n",
      "             The dataset to use for evaluation. If it is a [`~datasets.Dataset`], columns not accepted by the\n",
      "             `model.forward()` method are automatically removed. If it is a dictionary, it will evaluate on each\n",
      "             dataset prepending the dictionary key to the metric name.\n",
      "        tokenizer ([`PreTrainedTokenizerBase`], *optional*):\n",
      "            The tokenizer used to preprocess the data. If provided, will be used to automatically pad the inputs to the\n",
      "            maximum length when batching inputs, and it will be saved along the model to make it easier to rerun an\n",
      "            interrupted training or reuse the fine-tuned model.\n",
      "        model_init (`Callable[[], PreTrainedModel]`, *optional*):\n",
      "            A function that instantiates the model to be used. If provided, each call to [`~Trainer.train`] will start\n",
      "            from a new instance of the model as given by this function.\n",
      "\n",
      "            The function may have zero argument, or a single one containing the optuna/Ray Tune/SigOpt trial object, to\n",
      "            be able to choose different architectures according to hyper parameters (such as layer count, sizes of\n",
      "            inner layers, dropout probabilities etc).\n",
      "        compute_metrics (`Callable[[EvalPrediction], Dict]`, *optional*):\n",
      "            The function that will be used to compute metrics at evaluation. Must take a [`EvalPrediction`] and return\n",
      "            a dictionary string to metric values.\n",
      "        callbacks (List of [`TrainerCallback`], *optional*):\n",
      "            A list of callbacks to customize the training loop. Will add those to the list of default callbacks\n",
      "            detailed in [here](callback).\n",
      "\n",
      "            If you want to remove one of the default callbacks used, use the [`Trainer.remove_callback`] method.\n",
      "        optimizers (`Tuple[torch.optim.Optimizer, torch.optim.lr_scheduler.LambdaLR]`, *optional*, defaults to `(None, None)`):\n",
      "            A tuple containing the optimizer and the scheduler to use. Will default to an instance of [`AdamW`] on your\n",
      "            model and a scheduler given by [`get_linear_schedule_with_warmup`] controlled by `args`.\n",
      "        preprocess_logits_for_metrics (`Callable[[torch.Tensor, torch.Tensor], torch.Tensor]`, *optional*):\n",
      "            A function that preprocess the logits right before caching them at each evaluation step. Must take two\n",
      "            tensors, the logits and the labels, and return the logits once processed as desired. The modifications made\n",
      "            by this function will be reflected in the predictions received by `compute_metrics`.\n",
      "\n",
      "            Note that the labels (second parameter) will be `None` if the dataset does not have them.\n",
      "\n",
      "    Important attributes:\n",
      "\n",
      "        - **model** -- Always points to the core model. If using a transformers model, it will be a [`PreTrainedModel`]\n",
      "          subclass.\n",
      "        - **model_wrapped** -- Always points to the most external model in case one or more other modules wrap the\n",
      "          original model. This is the model that should be used for the forward pass. For example, under `DeepSpeed`,\n",
      "          the inner model is wrapped in `DeepSpeed` and then again in `torch.nn.DistributedDataParallel`. If the inner\n",
      "          model hasn't been wrapped, then `self.model_wrapped` is the same as `self.model`.\n",
      "        - **is_model_parallel** -- Whether or not a model has been switched to a model parallel mode (different from\n",
      "          data parallelism, this means some of the model layers are split on different GPUs).\n",
      "        - **place_model_on_device** -- Whether or not to automatically place the model on the device - it will be set\n",
      "          to `False` if model parallel or deepspeed is used, or if the default\n",
      "          `TrainingArguments.place_model_on_device` is overridden to return `False` .\n",
      "        - **is_in_train** -- Whether or not a model is currently running `train` (e.g. when `evaluate` is called while\n",
      "          in `train`)\n",
      "\n",
      "    \n",
      "__eq__: <slot wrapper '__eq__' of 'object' objects>\n",
      "__format__: <method '__format__' of 'object' objects>\n",
      "__ge__: <slot wrapper '__ge__' of 'object' objects>\n",
      "__getattribute__: <slot wrapper '__getattribute__' of 'object' objects>\n",
      "__gt__: <slot wrapper '__gt__' of 'object' objects>\n",
      "__hash__: <slot wrapper '__hash__' of 'object' objects>\n",
      "__init__: <function Trainer.__init__ at 0x29bcacdc0>\n",
      "__init_subclass__: <built-in method __init_subclass__ of type object at 0x29b79acf0>\n",
      "__le__: <slot wrapper '__le__' of 'object' objects>\n",
      "__lt__: <slot wrapper '__lt__' of 'object' objects>\n",
      "__module__: transformers.trainer\n",
      "__ne__: <slot wrapper '__ne__' of 'object' objects>\n",
      "__new__: <built-in method __new__ of type object at 0x104d1ab00>\n",
      "__reduce__: <method '__reduce__' of 'object' objects>\n",
      "__reduce_ex__: <method '__reduce_ex__' of 'object' objects>\n",
      "__repr__: <slot wrapper '__repr__' of 'object' objects>\n",
      "__setattr__: <slot wrapper '__setattr__' of 'object' objects>\n",
      "__sizeof__: <method '__sizeof__' of 'object' objects>\n",
      "__str__: <slot wrapper '__str__' of 'object' objects>\n",
      "__subclasshook__: <built-in method __subclasshook__ of type object at 0x29b79acf0>\n",
      "__weakref__: <attribute '__weakref__' of 'Trainer' objects>\n",
      "_activate_neftune: <function Trainer._activate_neftune at 0x29bcace50>\n",
      "_add_sm_patterns_to_gitignore: <function Trainer._add_sm_patterns_to_gitignore at 0x29bcb35e0>\n",
      "_deactivate_neftune: <function Trainer._deactivate_neftune at 0x29bcacee0>\n",
      "_finish_current_push: <function Trainer._finish_current_push at 0x29bcb33a0>\n",
      "_gather_and_numpify: <function Trainer._gather_and_numpify at 0x29bcb3550>\n",
      "_get_collator_with_removed_columns: <function Trainer._get_collator_with_removed_columns at 0x29bcae310>\n",
      "_get_eval_sampler: <function Trainer._get_eval_sampler at 0x29bcae4c0>\n",
      "_get_learning_rate: <function _get_learning_rate at 0x14c9e6940>\n",
      "_get_output_dir: <function Trainer._get_output_dir at 0x29bcaef70>\n",
      "_get_train_sampler: <function Trainer._get_train_sampler at 0x29bcae3a0>\n",
      "_hp_search_setup: <function Trainer._hp_search_setup at 0x29bcaea60>\n",
      "_inner_training_loop: <function Trainer._inner_training_loop at 0x29bcaeee0>\n",
      "_issue_warnings_after_load: <function Trainer._issue_warnings_after_load at 0x29bcb0160>\n",
      "_load_best_model: <function Trainer._load_best_model at 0x29bcb00d0>\n",
      "_load_from_checkpoint: <function Trainer._load_from_checkpoint at 0x29bcb0040>\n",
      "_load_optimizer_and_scheduler: <function Trainer._load_optimizer_and_scheduler at 0x29bcb04c0>\n",
      "_load_rng_state: <function Trainer._load_rng_state at 0x29bcb0280>\n",
      "_maybe_log_save_evaluate: <function Trainer._maybe_log_save_evaluate at 0x29bcb01f0>\n",
      "_move_model_to_device: <function Trainer._move_model_to_device at 0x29bcae160>\n",
      "_nested_gather: <function Trainer._nested_gather at 0x29bcb3040>\n",
      "_prepare_input: <function Trainer._prepare_input at 0x29bcb0670>\n",
      "_prepare_inputs: <function Trainer._prepare_inputs at 0x29bcb0700>\n",
      "_push_from_checkpoint: <function Trainer._push_from_checkpoint at 0x29bcb3310>\n",
      "_remove_unused_columns: <function Trainer._remove_unused_columns at 0x29bcae280>\n",
      "_report_to_hp_search: <function Trainer._report_to_hp_search at 0x29bcaeaf0>\n",
      "_rotate_checkpoints: <function Trainer._rotate_checkpoints at 0x29bcb0dc0>\n",
      "_save: <function Trainer._save at 0x29bcb0c10>\n",
      "_save_checkpoint: <function Trainer._save_checkpoint at 0x29bcb0310>\n",
      "_save_optimizer_and_scheduler: <function Trainer._save_optimizer_and_scheduler at 0x29bcb0430>\n",
      "_save_rng_state: <function Trainer._save_rng_state at 0x29bcb03a0>\n",
      "_save_tpu: <function Trainer._save_tpu at 0x29bcb0b80>\n",
      "_set_signature_columns_if_needed: <function Trainer._set_signature_columns_if_needed at 0x29bcae1f0>\n",
      "_sorted_checkpoints: <function Trainer._sorted_checkpoints at 0x29bcb0d30>\n",
      "_tune_save_checkpoint: <function Trainer._tune_save_checkpoint at 0x29bcaeb80>\n",
      "_wrap_model: <function Trainer._wrap_model at 0x29bcaedc0>\n",
      "add_callback: <function Trainer.add_callback at 0x29bcacf70>\n",
      "autocast_smart_context_manager: <function Trainer.autocast_smart_context_manager at 0x29bcb0820>\n",
      "call_model_init: <function Trainer.call_model_init at 0x29bcaec10>\n",
      "compute_loss: <function Trainer.compute_loss at 0x29bcb0940>\n",
      "compute_loss_context_manager: <function Trainer.compute_loss_context_manager at 0x29bcb0790>\n",
      "create_accelerator_and_postprocess: <function Trainer.create_accelerator_and_postprocess at 0x29bcb3670>\n",
      "create_model_card: <function Trainer.create_model_card at 0x29bcb3280>\n",
      "create_optimizer: <function Trainer.create_optimizer at 0x29bcae790>\n",
      "create_optimizer_and_scheduler: <function Trainer.create_optimizer_and_scheduler at 0x29bcae670>\n",
      "create_scheduler: <function Trainer.create_scheduler at 0x29bcae8b0>\n",
      "evaluate: <function Trainer.evaluate at 0x29bcb0e50>\n",
      "evaluation_loop: <function Trainer.evaluation_loop at 0x29bcb0f70>\n",
      "floating_point_ops: <function Trainer.floating_point_ops at 0x29bcb3160>\n",
      "get_decay_parameter_names: <function Trainer.get_decay_parameter_names at 0x29bcae700>\n",
      "get_eval_dataloader: <function Trainer.get_eval_dataloader at 0x29bcae550>\n",
      "get_optimizer_cls_and_kwargs: <function Trainer.get_optimizer_cls_and_kwargs at 0x29bcae820>\n",
      "get_test_dataloader: <function Trainer.get_test_dataloader at 0x29bcae5e0>\n",
      "get_train_dataloader: <function Trainer.get_train_dataloader at 0x29bcae430>\n",
      "hyperparameter_search: <function Trainer.hyperparameter_search at 0x29bcb0550>\n",
      "init_hf_repo: <function Trainer.init_hf_repo at 0x29bcb31f0>\n",
      "ipex_optimize_model: <function Trainer.ipex_optimize_model at 0x29bcaed30>\n",
      "is_local_process_zero: <function Trainer.is_local_process_zero at 0x29bcb09d0>\n",
      "is_world_process_zero: <function Trainer.is_world_process_zero at 0x29bcb0a60>\n",
      "log: <function Trainer.log at 0x29bcb05e0>\n",
      "log_metrics: <function log_metrics at 0x14c9e7280>\n",
      "metrics_format: <function metrics_format at 0x14c9e71f0>\n",
      "num_examples: <function Trainer.num_examples at 0x29bcae940>\n",
      "num_tokens: <function Trainer.num_tokens at 0x29bcae9d0>\n",
      "pop_callback: <function Trainer.pop_callback at 0x29bcae040>\n",
      "predict: <function Trainer.predict at 0x29bcb0ee0>\n",
      "prediction_loop: <function Trainer.prediction_loop at 0x29bcb34c0>\n",
      "prediction_step: <function Trainer.prediction_step at 0x29bcb30d0>\n",
      "propagate_args_to_deepspeed: <function Trainer.propagate_args_to_deepspeed at 0x29bcb3700>\n",
      "push_to_hub: <function Trainer.push_to_hub at 0x29bcb3430>\n",
      "remove_callback: <function Trainer.remove_callback at 0x29bcae0d0>\n",
      "save_metrics: <function save_metrics at 0x14c9e7310>\n",
      "save_model: <function Trainer.save_model at 0x29bcb0af0>\n",
      "save_state: <function save_state at 0x14c9e73a0>\n",
      "store_flos: <function Trainer.store_flos at 0x29bcb0ca0>\n",
      "torch_jit_model_eval: <function Trainer.torch_jit_model_eval at 0x29bcaeca0>\n",
      "train: <function Trainer.train at 0x29bcaee50>\n",
      "training_step: <function Trainer.training_step at 0x29bcb08b0>\n"
     ]
    }
   ],
   "source": [
    "trainer_all_members = get_members_of_function_or_method(Trainer, predicate=None)\n",
    "loop_through_members(trainer_all_members)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Retrieve All Methods of a Class\n",
    "\n",
    "There are a few ways to do it."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Using `__dict__`"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_ChildClass__private_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__doc__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__module__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_protected_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'read_only_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'static_method'</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_ChildClass__private_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__doc__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__init__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__module__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__str__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_protected_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'read_only_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'static_method'\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "child_class_methods_using_dict = list(ChildClass.__dict__.keys())\n",
    "pprint(sorted(child_class_methods_using_dict))\n",
    "\n",
    "assert 'parent_method' not in child_class_methods_using_dict\n",
    "assert 'read_only_attr' in child_class_methods_using_dict\n",
    "assert 'class_method' in child_class_methods_using_dict"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Notice that the parent class methods are not included!"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"color: #00ff00; text-decoration-color: #00ff00; font-style: italic\">True</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[3;92mTrue\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "pprint(instance_child.__class__.__dict__.keys() == ChildClass.__dict__.keys())"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Using `vars`\n",
    "\n",
    "`vars` and `__dict__` are equivalent, but people are preferring the former due\n",
    "to some efficiency reasons, which can be found [in this post](https://stackoverflow.com/questions/21297203/use-dict-or-vars)."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_ChildClass__private_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__doc__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__module__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_protected_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'read_only_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'static_method'</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_ChildClass__private_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__doc__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__init__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__module__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__str__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_protected_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'read_only_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'static_method'\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "child_class_methods_using_vars = list(vars(ChildClass).keys())\n",
    "pprint(sorted(child_class_methods_using_vars))\n",
    "\n",
    "assert 'parent_method' not in child_class_methods_using_vars\n",
    "assert 'read_only_attr' in child_class_methods_using_vars\n",
    "assert 'class_method' in child_class_methods_using_vars\n",
    "\n",
    "assert set(child_class_methods_using_dict) == set(child_class_methods_using_vars)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Using `dir`\n",
    "\n",
    "To include the base/parent class methods, we can use `dir` instead."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_ChildClass__private_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__class__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__delattr__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__dict__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__dir__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__doc__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__eq__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__format__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__ge__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__getattribute__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__gt__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__hash__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__init_subclass__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__le__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__lt__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__module__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__ne__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__new__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce_ex__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__repr__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__setattr__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__sizeof__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__subclasshook__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__weakref__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_protected_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'parent_class_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'parent_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'read_only_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'static_method'</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_ChildClass__private_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__class__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__delattr__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__dict__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__dir__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__doc__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__eq__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__format__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__ge__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__getattribute__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__gt__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__hash__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__init__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__init_subclass__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__le__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__lt__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__module__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__ne__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__new__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__reduce__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__reduce_ex__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__repr__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__setattr__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__sizeof__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__str__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__subclasshook__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__weakref__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_protected_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'parent_class_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'parent_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'read_only_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'static_method'\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "child_class_methods_using_dir = list(dir(ChildClass))\n",
    "pprint(sorted(child_class_methods_using_dir))\n",
    "\n",
    "assert 'parent_method' in child_class_methods_using_dir\n",
    "assert 'read_only_attr' in child_class_methods_using_dir\n",
    "assert 'class_method' in child_class_methods_using_dir"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Using `inspect.getmembers`\n",
    "\n",
    "We use `inspect.getmembers` to get all members of a class, and then filter out\n",
    "via the predicate `inspect.isroutine`, a stronger filter than `inspect.isfunction`\n",
    "or `inspect.ismethod`.\n",
    "\n",
    "We attach the source code of `inspect.isroutine` here for reference.\n",
    "\n",
    "```python\n",
    "def isroutine(object):\n",
    "    \"\"\"Return true if the object is any kind of function or method.\"\"\"\n",
    "    return (isbuiltin(object)\n",
    "            or isfunction(object)\n",
    "            or ismethod(object)\n",
    "            or ismethoddescriptor(object))\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__delattr__'</span>, <span style=\"font-weight: bold\">&lt;</span><span style=\"color: #ff00ff; text-decoration-color: #ff00ff; font-weight: bold\">slot</span><span style=\"color: #000000; text-decoration-color: #000000\"> wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__delattr__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__dir__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__dir__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__eq__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__eq__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__format__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__format__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__ge__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__ge__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__getattribute__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__getattribute__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__gt__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__gt__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__hash__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__hash__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ChildClass.__init__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce64c0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__init_subclass__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;built-in method __init_subclass__ of type object at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b7a5190</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__le__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__le__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__lt__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__lt__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__ne__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__ne__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__new__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;built-in method __new__ of type object at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x104d1ab00</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce_ex__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce_ex__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__repr__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__repr__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__setattr__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__setattr__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__sizeof__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__sizeof__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ChildClass.__str__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce68b0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__subclasshook__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;built-in method __subclasshook__ of type object at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b7a5190</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'class_method'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;bound method ChildClass.class_method of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'__main__.ChildClass'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_method'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ChildClass.instance_method at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce6700</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'parent_method'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ParentClass.parent_method at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce65e0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'static_method'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ChildClass.static_method at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce6820</span><span style=\"font-weight: bold\">&gt;)</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1m(\u001b[0m\u001b[32m'__delattr__'\u001b[0m, \u001b[1m<\u001b[0m\u001b[1;95mslot\u001b[0m\u001b[39m wrapper \u001b[0m\u001b[32m'__delattr__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__dir__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__dir__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__eq__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__eq__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__format__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__format__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__ge__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__ge__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__getattribute__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__getattribute__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__gt__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__gt__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__hash__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__hash__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__init__'\u001b[0m\u001b[39m, <function ChildClass.__init__ at \u001b[0m\u001b[1;36m0x29bce64c0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__init_subclass__'\u001b[0m\u001b[39m, <built-in method __init_subclass__ of type object at \u001b[0m\u001b[1;36m0x29b7a5190\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__le__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__le__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__lt__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__lt__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__ne__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__ne__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__new__'\u001b[0m\u001b[39m, <built-in method __new__ of type object at \u001b[0m\u001b[1;36m0x104d1ab00\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__reduce__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__reduce__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__reduce_ex__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__reduce_ex__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__repr__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__repr__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__setattr__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__setattr__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__sizeof__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__sizeof__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__str__'\u001b[0m\u001b[39m, <function ChildClass.__str__ at \u001b[0m\u001b[1;36m0x29bce68b0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__subclasshook__'\u001b[0m\u001b[39m, <built-in method __subclasshook__ of type object at \u001b[0m\u001b[1;36m0x29b7a5190\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'class_method'\u001b[0m\u001b[39m, <bound method ChildClass.class_method of <class \u001b[0m\u001b[32m'__main__.ChildClass'\u001b[0m\u001b[39m>>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'instance_method'\u001b[0m\u001b[39m, <function ChildClass.instance_method at \u001b[0m\u001b[1;36m0x29bce6700\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'parent_method'\u001b[0m\u001b[39m, <function ParentClass.parent_method at \u001b[0m\u001b[1;36m0x29bce65e0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'static_method'\u001b[0m\u001b[39m, <function ChildClass.static_method at \u001b[0m\u001b[1;36m0x29bce6820\u001b[0m\u001b[1m>\u001b[0m\u001b[1m)\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "predicate = inspect.isroutine\n",
    "child_class_methods_using_getmembers = list(get_members_of_function_or_method(ChildClass, predicate=predicate))\n",
    "\n",
    "pprint(sorted(child_class_methods_using_getmembers))"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Of course, the reason to retrieve all methods is a convenience if we want to\n",
    "inspect all methods at once. And if we can obtain all methods, we can then\n",
    "iteratively inspect each method's signature.\n",
    "\n",
    "### Method Resolution Order\n",
    "\n",
    "The above examples do not take into account complicated cases, such as when\n",
    "the class is a subclass of **multiple** classes, in which case if you just\n",
    "print out the methods of the class, you will have a hard time to know which\n",
    "methods are from which class. You can do so via more filtering, but this is\n",
    "beyond the scope of this notebook. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__call__'</span>, <span style=\"font-weight: bold\">&lt;</span><span style=\"color: #ff00ff; text-decoration-color: #ff00ff; font-weight: bold\">function</span><span style=\"color: #000000; text-decoration-color: #000000\"> Module._wrapped_call_impl at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__delattr__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__delattr__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__dir__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__dir__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e4160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__eq__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__eq__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__format__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__format__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__ge__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__ge__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__getattr__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__getattr__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__getattribute__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__getattribute__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__getstate__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__getstate__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__gt__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__gt__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__hash__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__hash__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.__init__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce45e0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__init_subclass__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;built-in method __init_subclass__ of type object at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b7a2e40</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__le__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__le__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__lt__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__lt__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__ne__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__ne__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__new__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;built-in method __new__ of type object at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x104d1ab00</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce_ex__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__reduce_ex__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__repr__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__repr__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e40d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__setattr__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__setattr__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e01f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__setstate__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.__setstate__ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e00d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__sizeof__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;method </span><span style=\"color: #008000; text-decoration-color: #008000\">'__sizeof__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;slot wrapper </span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span><span style=\"color: #000000; text-decoration-color: #000000\"> of </span><span style=\"color: #008000; text-decoration-color: #008000\">'object'</span><span style=\"color: #000000; text-decoration-color: #000000\"> objects&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'__subclasshook__'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;built-in method __subclasshook__ of type object at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b7a2e40</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_apply'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._apply at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_autoset_attn_implementation'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel._autoset_attn_implementation of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_backward_compatibility_gradient_checkpointing'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function PreTrainedModel._backward_compatibility_gradient_checkpointing at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed670</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_call_impl'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._call_impl at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddf70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_check_and_enable_flash_attn_2'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel._check_and_enable_flash_attn_2 of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_check_and_enable_sdpa'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel._check_and_enable_sdpa of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_convert_head_mask_to_5d'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin._convert_head_mask_to_5d at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed1f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_copy_lm_head_original_to_resized'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function PreTrainedModel._copy_lm_head_original_to_resized at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef430</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_create_repo'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PushToHubMixin._create_repo at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x14b2a79d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_dispatch_accelerate_model'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin._dispatch_accelerate_model at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2e50</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_expand_inputs_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._expand_inputs_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4430</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_extract_past_from_model_output'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function GenerationMixin._extract_past_from_model_output at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b44c0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_from_config'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel._from_config of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_backward_hooks'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._get_backward_hooks at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddb80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_backward_pre_hooks'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._get_backward_pre_hooks at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddc10</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_candidate_generator'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._get_candidate_generator at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4670</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_decoder_start_token_id'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._get_decoder_start_token_id at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b43a0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_files_timestamps'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PushToHubMixin._get_files_timestamps at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x14b2a7a60</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_generation_mode'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._get_generation_mode at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4790</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_logits_processor'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._get_logits_processor at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4820</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_logits_warper'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._get_logits_warper at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4700</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_name'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._get_name at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0f70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_no_split_modules'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._get_no_split_modules at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_resized_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._get_resized_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef310</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_resized_lm_head'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._get_resized_lm_head at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef3a0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_get_stopping_criteria'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._get_stopping_criteria at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b48b0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_hook_rss_memory_post_forward'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin._hook_rss_memory_post_forward at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ebca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_hook_rss_memory_pre_forward'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin._hook_rss_memory_pre_forward at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ebc10</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_init_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2PreTrainedModel._init_weights at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bcdfe50</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_initialize_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._initialize_weights at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3edee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_load_from_state_dict'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._load_from_state_dict at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0670</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_load_pretrained_model'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel._load_pretrained_model of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_load_pretrained_model_low_mem'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._load_pretrained_model_low_mem at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_maybe_initialize_input_ids_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function GenerationMixin._maybe_initialize_input_ids_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_maybe_warn_non_full_backward_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._maybe_warn_non_full_backward_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_merge_criteria_processor_list'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._merge_criteria_processor_list at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4940</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_named_members'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._named_members at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0790</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_prepare_attention_mask_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function GenerationMixin._prepare_attention_mask_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b41f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_prepare_decoder_input_ids_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function GenerationMixin._prepare_decoder_input_ids_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4310</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_prepare_encoder_decoder_kwargs_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function GenerationMixin._prepare_encoder_decoder_kwargs_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_prepare_model_inputs'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._prepare_model_inputs at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b40d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_register_load_state_dict_pre_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._register_load_state_dict_pre_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0550</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_register_state_dict_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._register_state_dict_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0310</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_reorder_cache'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel._reorder_cache at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce49d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_replicate_for_data_parallel'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._replicate_for_data_parallel at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e41f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_resize_token_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._resize_token_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_save_to_state_dict'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._save_to_state_dict at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0430</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_set_default_torch_dtype'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel._set_default_torch_dtype of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_set_gradient_checkpointing'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._set_gradient_checkpointing at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef790</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_slow_forward'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._slow_forward at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dde50</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_temporary_reorder_cache'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._temporary_reorder_cache at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4f70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_tie_encoder_decoder_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._tie_encoder_decoder_weights at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_tie_or_clone_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel._tie_or_clone_weights at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef0d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_update_model_kwargs_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function GenerationMixin._update_model_kwargs_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4550</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_upload_modified_files'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PushToHubMixin._upload_modified_files at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x14b2a7af0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_validate_generated_length'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._validate_generated_length at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4b80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_validate_model_class'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._validate_model_class at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4a60</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_validate_model_kwargs'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin._validate_model_kwargs at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4af0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'_wrapped_call_impl'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module._wrapped_call_impl at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'active_adapter'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.active_adapter at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2d30</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'active_adapters'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.active_adapters at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2ca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'add_adapter'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.add_adapter at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2a60</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'add_memory_hooks'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.add_memory_hooks at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ebd30</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'add_model_tags'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.add_model_tags at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed700</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'add_module'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.add_module at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463d6e50</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'apply'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.apply at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd310</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'assisted_decoding'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.assisted_decoding at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'beam_sample'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.beam_sample at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b20d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'beam_search'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.beam_search at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'bfloat16'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.bfloat16 at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd820</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'buffers'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.buffers at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0940</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'can_generate'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel.can_generate of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'children'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.children at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0a60</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'compile'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.compile at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e4280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'compute_transition_scores'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.compute_transition_scores at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b49d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'constrained_beam_search'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.constrained_beam_search at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b21f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'contrastive_search'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.contrastive_search at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4dc0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'cpu'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.cpu at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd550</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'create_extended_attention_mask_for_decoder'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function ModuleUtilsMixin.create_extended_attention_mask_for_decoder at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'cuda'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.cuda at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efaf0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'deparallelize'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.deparallelize at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce4820</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'disable_adapters'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.disable_adapters at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2b80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'disable_input_require_grads'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.disable_input_require_grads at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3edc10</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'double'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.double at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd700</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'enable_adapters'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.enable_adapters at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2c10</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'enable_input_require_grads'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.enable_input_require_grads at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3edb80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'estimate_tokens'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.estimate_tokens at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed310</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'eval'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.eval at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0d30</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'extra_repr'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.extra_repr at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e4040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'float'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.float at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'floating_point_ops'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.floating_point_ops at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed3a0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'forward'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.forward at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce4af0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'from_pretrained'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel.from_pretrained of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'generate'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.generate at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4ca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_adapter_state_dict'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.get_adapter_state_dict at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2dc0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_buffer'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.get_buffer at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd0d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_extended_attention_mask'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.get_extended_attention_mask at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed0d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_extra_state'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.get_extra_state at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_head_mask'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.get_head_mask at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_input_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.get_input_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3edca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_memory_footprint'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.get_memory_footprint at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efa60</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_output_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.get_output_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce4670</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_parameter'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.get_parameter at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_position_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.get_position_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef550</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'get_submodule'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.get_submodule at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463d6f70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'gradient_checkpointing_disable'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.gradient_checkpointing_disable at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef820</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'gradient_checkpointing_enable'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.gradient_checkpointing_enable at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef700</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'greedy_search'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.greedy_search at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4e50</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'group_beam_search'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.group_beam_search at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'half'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.half at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efc10</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'init_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.init_weights at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef5e0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'invert_attention_mask'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.invert_attention_mask at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ebf70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'ipu'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.ipu at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd430</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'load_adapter'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.load_adapter at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b29d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'load_state_dict'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.load_state_dict at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0700</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'load_tf_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function load_tf_weights_in_gpt2 at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bcb3f70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'modules'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.modules at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0b80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'named_buffers'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.named_buffers at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e09d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'named_children'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.named_children at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0af0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'named_modules'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.named_modules at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0c10</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'named_parameters'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.named_parameters at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e08b0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'num_parameters'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.num_parameters at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed280</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'parallelize'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.parallelize at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce4790</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'parameters'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.parameters at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0820</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'post_init'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.post_init at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ed5e0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'prepare_inputs_for_generation'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.prepare_inputs_for_generation at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce4940</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'prune_heads'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.prune_heads at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef670</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'push_to_hub'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PushToHubMixin.push_to_hub at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ebb80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_backward_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_backward_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dda60</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_buffer'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_buffer at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463d6d30</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'register_for_auto_class'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;bound method PreTrainedModel.register_for_auto_class of &lt;class </span><span style=\"color: #008000; text-decoration-color: #008000\">'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_forward_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_forward_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dddc0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_forward_pre_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_forward_pre_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddd30</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_full_backward_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_full_backward_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463ddaf0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_full_backward_pre_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_full_backward_pre_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd9d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_load_state_dict_post_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_load_state_dict_post_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e05e0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_module'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_module at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463d6ee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_parameter'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_parameter at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463d6dc0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'register_state_dict_pre_hook'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.register_state_dict_pre_hook at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e03a0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'requires_grad_'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.requires_grad_ at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0dc0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'reset_memory_hooks_state'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function ModuleUtilsMixin.reset_memory_hooks_state at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ebdc0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'resize_position_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.resize_position_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef4c0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'resize_token_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.resize_token_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef1f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'retrieve_modules_from_names'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.retrieve_modules_from_names at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efe50</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'reverse_bettertransformer'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.reverse_bettertransformer at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3f20d0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'sample'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GenerationMixin.sample at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b4ee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'save_pretrained'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.save_pretrained at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3ef940</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'set_adapter'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PeftAdapterMixin.set_adapter at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3b2af0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'set_extra_state'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.set_extra_state at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd1f0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'set_input_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.set_input_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3edd30</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'set_output_embeddings'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function GPT2LMHeadModel.set_output_embeddings at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29bce48b0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'share_memory'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.share_memory at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0ee0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'state_dict'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.state_dict at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e04c0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'tie_weights'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.tie_weights at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3edf70</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'to'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.to at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3efb80</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'to_bettertransformer'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function PreTrainedModel.to_bettertransformer at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3f2040</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'to_empty'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.to_empty at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd8b0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'train'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.train at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0ca0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'type'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.type at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd5e0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #008000; text-decoration-color: #008000\">'warn_if_padding_and_no_attention_mask'</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   │   </span><span style=\"color: #000000; text-decoration-color: #000000\">&lt;function PreTrainedModel.warn_if_padding_and_no_attention_mask at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x29b3f2160</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'xpu'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.xpu at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463dd4c0</span><span style=\"color: #000000; text-decoration-color: #000000\">&gt;</span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">)</span><span style=\"color: #000000; text-decoration-color: #000000\">,</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #000000; text-decoration-color: #000000; font-weight: bold\">(</span><span style=\"color: #008000; text-decoration-color: #008000\">'zero_grad'</span><span style=\"color: #000000; text-decoration-color: #000000\">, &lt;function Module.zero_grad at </span><span style=\"color: #008080; text-decoration-color: #008080; font-weight: bold\">0x1463e0e50</span><span style=\"font-weight: bold\">&gt;)</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1m(\u001b[0m\u001b[32m'__call__'\u001b[0m, \u001b[1m<\u001b[0m\u001b[1;95mfunction\u001b[0m\u001b[39m Module._wrapped_call_impl at \u001b[0m\u001b[1;36m0x1463ddee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__delattr__'\u001b[0m\u001b[39m, <function Module.__delattr__ at \u001b[0m\u001b[1;36m0x1463e0280\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__dir__'\u001b[0m\u001b[39m, <function Module.__dir__ at \u001b[0m\u001b[1;36m0x1463e4160\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__eq__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__eq__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__format__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__format__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__ge__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__ge__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__getattr__'\u001b[0m\u001b[39m, <function Module.__getattr__ at \u001b[0m\u001b[1;36m0x1463e0160\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__getattribute__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__getattribute__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__getstate__'\u001b[0m\u001b[39m, <function Module.__getstate__ at \u001b[0m\u001b[1;36m0x1463e0040\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__gt__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__gt__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__hash__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__hash__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__init__'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.__init__ at \u001b[0m\u001b[1;36m0x29bce45e0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__init_subclass__'\u001b[0m\u001b[39m, <built-in method __init_subclass__ of type object at \u001b[0m\u001b[1;36m0x29b7a2e40\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__le__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__le__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__lt__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__lt__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__ne__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__ne__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__new__'\u001b[0m\u001b[39m, <built-in method __new__ of type object at \u001b[0m\u001b[1;36m0x104d1ab00\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__reduce__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__reduce__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__reduce_ex__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__reduce_ex__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__repr__'\u001b[0m\u001b[39m, <function Module.__repr__ at \u001b[0m\u001b[1;36m0x1463e40d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__setattr__'\u001b[0m\u001b[39m, <function Module.__setattr__ at \u001b[0m\u001b[1;36m0x1463e01f0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__setstate__'\u001b[0m\u001b[39m, <function Module.__setstate__ at \u001b[0m\u001b[1;36m0x1463e00d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__sizeof__'\u001b[0m\u001b[39m, <method \u001b[0m\u001b[32m'__sizeof__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__str__'\u001b[0m\u001b[39m, <slot wrapper \u001b[0m\u001b[32m'__str__'\u001b[0m\u001b[39m of \u001b[0m\u001b[32m'object'\u001b[0m\u001b[39m objects>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'__subclasshook__'\u001b[0m\u001b[39m, <built-in method __subclasshook__ of type object at \u001b[0m\u001b[1;36m0x29b7a2e40\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_apply'\u001b[0m\u001b[39m, <function Module._apply at \u001b[0m\u001b[1;36m0x1463dd280\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_autoset_attn_implementation'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel._autoset_attn_implementation of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_backward_compatibility_gradient_checkpointing'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function PreTrainedModel._backward_compatibility_gradient_checkpointing at \u001b[0m\u001b[1;36m0x29b3ed670\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_call_impl'\u001b[0m\u001b[39m, <function Module._call_impl at \u001b[0m\u001b[1;36m0x1463ddf70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_check_and_enable_flash_attn_2'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel._check_and_enable_flash_attn_2 of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_check_and_enable_sdpa'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel._check_and_enable_sdpa of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_convert_head_mask_to_5d'\u001b[0m\u001b[39m, <function ModuleUtilsMixin._convert_head_mask_to_5d at \u001b[0m\u001b[1;36m0x29b3ed1f0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_copy_lm_head_original_to_resized'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function PreTrainedModel._copy_lm_head_original_to_resized at \u001b[0m\u001b[1;36m0x29b3ef430\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_create_repo'\u001b[0m\u001b[39m, <function PushToHubMixin._create_repo at \u001b[0m\u001b[1;36m0x14b2a79d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_dispatch_accelerate_model'\u001b[0m\u001b[39m, <function PeftAdapterMixin._dispatch_accelerate_model at \u001b[0m\u001b[1;36m0x29b3b2e50\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_expand_inputs_for_generation'\u001b[0m\u001b[39m, <function GenerationMixin._expand_inputs_for_generation at \u001b[0m\u001b[1;36m0x29b3b4430\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_extract_past_from_model_output'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function GenerationMixin._extract_past_from_model_output at \u001b[0m\u001b[1;36m0x29b3b44c0\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_from_config'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel._from_config of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_backward_hooks'\u001b[0m\u001b[39m, <function Module._get_backward_hooks at \u001b[0m\u001b[1;36m0x1463ddb80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_backward_pre_hooks'\u001b[0m\u001b[39m, <function Module._get_backward_pre_hooks at \u001b[0m\u001b[1;36m0x1463ddc10\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_candidate_generator'\u001b[0m\u001b[39m, <function GenerationMixin._get_candidate_generator at \u001b[0m\u001b[1;36m0x29b3b4670\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_decoder_start_token_id'\u001b[0m\u001b[39m, <function GenerationMixin._get_decoder_start_token_id at \u001b[0m\u001b[1;36m0x29b3b43a0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_files_timestamps'\u001b[0m\u001b[39m, <function PushToHubMixin._get_files_timestamps at \u001b[0m\u001b[1;36m0x14b2a7a60\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_generation_mode'\u001b[0m\u001b[39m, <function GenerationMixin._get_generation_mode at \u001b[0m\u001b[1;36m0x29b3b4790\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_logits_processor'\u001b[0m\u001b[39m, <function GenerationMixin._get_logits_processor at \u001b[0m\u001b[1;36m0x29b3b4820\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_logits_warper'\u001b[0m\u001b[39m, <function GenerationMixin._get_logits_warper at \u001b[0m\u001b[1;36m0x29b3b4700\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_name'\u001b[0m\u001b[39m, <function Module._get_name at \u001b[0m\u001b[1;36m0x1463e0f70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_no_split_modules'\u001b[0m\u001b[39m, <function PreTrainedModel._get_no_split_modules at \u001b[0m\u001b[1;36m0x29b3ef160\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_resized_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel._get_resized_embeddings at \u001b[0m\u001b[1;36m0x29b3ef310\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_resized_lm_head'\u001b[0m\u001b[39m, <function PreTrainedModel._get_resized_lm_head at \u001b[0m\u001b[1;36m0x29b3ef3a0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_get_stopping_criteria'\u001b[0m\u001b[39m, <function GenerationMixin._get_stopping_criteria at \u001b[0m\u001b[1;36m0x29b3b48b0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_hook_rss_memory_post_forward'\u001b[0m\u001b[39m, <function ModuleUtilsMixin._hook_rss_memory_post_forward at \u001b[0m\u001b[1;36m0x29b3ebca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_hook_rss_memory_pre_forward'\u001b[0m\u001b[39m, <function ModuleUtilsMixin._hook_rss_memory_pre_forward at \u001b[0m\u001b[1;36m0x29b3ebc10\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_init_weights'\u001b[0m\u001b[39m, <function GPT2PreTrainedModel._init_weights at \u001b[0m\u001b[1;36m0x29bcdfe50\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_initialize_weights'\u001b[0m\u001b[39m, <function PreTrainedModel._initialize_weights at \u001b[0m\u001b[1;36m0x29b3edee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_load_from_state_dict'\u001b[0m\u001b[39m, <function Module._load_from_state_dict at \u001b[0m\u001b[1;36m0x1463e0670\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_load_pretrained_model'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel._load_pretrained_model of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_load_pretrained_model_low_mem'\u001b[0m\u001b[39m, <function PreTrainedModel._load_pretrained_model_low_mem at \u001b[0m\u001b[1;36m0x29b3efee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_maybe_initialize_input_ids_for_generation'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function GenerationMixin._maybe_initialize_input_ids_for_generation at \u001b[0m\u001b[1;36m0x29b3b4160\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_maybe_warn_non_full_backward_hook'\u001b[0m\u001b[39m, <function Module._maybe_warn_non_full_backward_hook at \u001b[0m\u001b[1;36m0x1463ddca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_merge_criteria_processor_list'\u001b[0m\u001b[39m, <function GenerationMixin._merge_criteria_processor_list at \u001b[0m\u001b[1;36m0x29b3b4940\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_named_members'\u001b[0m\u001b[39m, <function Module._named_members at \u001b[0m\u001b[1;36m0x1463e0790\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_prepare_attention_mask_for_generation'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function GenerationMixin._prepare_attention_mask_for_generation at \u001b[0m\u001b[1;36m0x29b3b41f0\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_prepare_decoder_input_ids_for_generation'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function GenerationMixin._prepare_decoder_input_ids_for_generation at \u001b[0m\u001b[1;36m0x29b3b4310\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_prepare_encoder_decoder_kwargs_for_generation'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function GenerationMixin._prepare_encoder_decoder_kwargs_for_generation at \u001b[0m\u001b[1;36m0x29b3b4280\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_prepare_model_inputs'\u001b[0m\u001b[39m, <function GenerationMixin._prepare_model_inputs at \u001b[0m\u001b[1;36m0x29b3b40d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_register_load_state_dict_pre_hook'\u001b[0m\u001b[39m, <function Module._register_load_state_dict_pre_hook at \u001b[0m\u001b[1;36m0x1463e0550\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_register_state_dict_hook'\u001b[0m\u001b[39m, <function Module._register_state_dict_hook at \u001b[0m\u001b[1;36m0x1463e0310\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_reorder_cache'\u001b[0m\u001b[39m, <function GPT2LMHeadModel._reorder_cache at \u001b[0m\u001b[1;36m0x29bce49d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_replicate_for_data_parallel'\u001b[0m\u001b[39m, <function Module._replicate_for_data_parallel at \u001b[0m\u001b[1;36m0x1463e41f0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_resize_token_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel._resize_token_embeddings at \u001b[0m\u001b[1;36m0x29b3ef280\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_save_to_state_dict'\u001b[0m\u001b[39m, <function Module._save_to_state_dict at \u001b[0m\u001b[1;36m0x1463e0430\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_set_default_torch_dtype'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel._set_default_torch_dtype of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_set_gradient_checkpointing'\u001b[0m\u001b[39m, <function PreTrainedModel._set_gradient_checkpointing at \u001b[0m\u001b[1;36m0x29b3ef790\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_slow_forward'\u001b[0m\u001b[39m, <function Module._slow_forward at \u001b[0m\u001b[1;36m0x1463dde50\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_temporary_reorder_cache'\u001b[0m\u001b[39m, <function GenerationMixin._temporary_reorder_cache at \u001b[0m\u001b[1;36m0x29b3b4f70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_tie_encoder_decoder_weights'\u001b[0m\u001b[39m, <function PreTrainedModel._tie_encoder_decoder_weights at \u001b[0m\u001b[1;36m0x29b3ef040\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_tie_or_clone_weights'\u001b[0m\u001b[39m, <function PreTrainedModel._tie_or_clone_weights at \u001b[0m\u001b[1;36m0x29b3ef0d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'_update_model_kwargs_for_generation'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function GenerationMixin._update_model_kwargs_for_generation at \u001b[0m\u001b[1;36m0x29b3b4550\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_upload_modified_files'\u001b[0m\u001b[39m, <function PushToHubMixin._upload_modified_files at \u001b[0m\u001b[1;36m0x14b2a7af0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_validate_generated_length'\u001b[0m\u001b[39m, <function GenerationMixin._validate_generated_length at \u001b[0m\u001b[1;36m0x29b3b4b80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_validate_model_class'\u001b[0m\u001b[39m, <function GenerationMixin._validate_model_class at \u001b[0m\u001b[1;36m0x29b3b4a60\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_validate_model_kwargs'\u001b[0m\u001b[39m, <function GenerationMixin._validate_model_kwargs at \u001b[0m\u001b[1;36m0x29b3b4af0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'_wrapped_call_impl'\u001b[0m\u001b[39m, <function Module._wrapped_call_impl at \u001b[0m\u001b[1;36m0x1463ddee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'active_adapter'\u001b[0m\u001b[39m, <function PeftAdapterMixin.active_adapter at \u001b[0m\u001b[1;36m0x29b3b2d30\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'active_adapters'\u001b[0m\u001b[39m, <function PeftAdapterMixin.active_adapters at \u001b[0m\u001b[1;36m0x29b3b2ca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'add_adapter'\u001b[0m\u001b[39m, <function PeftAdapterMixin.add_adapter at \u001b[0m\u001b[1;36m0x29b3b2a60\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'add_memory_hooks'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.add_memory_hooks at \u001b[0m\u001b[1;36m0x29b3ebd30\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'add_model_tags'\u001b[0m\u001b[39m, <function PreTrainedModel.add_model_tags at \u001b[0m\u001b[1;36m0x29b3ed700\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'add_module'\u001b[0m\u001b[39m, <function Module.add_module at \u001b[0m\u001b[1;36m0x1463d6e50\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'apply'\u001b[0m\u001b[39m, <function Module.apply at \u001b[0m\u001b[1;36m0x1463dd310\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'assisted_decoding'\u001b[0m\u001b[39m, <function GenerationMixin.assisted_decoding at \u001b[0m\u001b[1;36m0x29b3b2280\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'beam_sample'\u001b[0m\u001b[39m, <function GenerationMixin.beam_sample at \u001b[0m\u001b[1;36m0x29b3b20d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'beam_search'\u001b[0m\u001b[39m, <function GenerationMixin.beam_search at \u001b[0m\u001b[1;36m0x29b3b2040\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'bfloat16'\u001b[0m\u001b[39m, <function Module.bfloat16 at \u001b[0m\u001b[1;36m0x1463dd820\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'buffers'\u001b[0m\u001b[39m, <function Module.buffers at \u001b[0m\u001b[1;36m0x1463e0940\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'can_generate'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel.can_generate of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'children'\u001b[0m\u001b[39m, <function Module.children at \u001b[0m\u001b[1;36m0x1463e0a60\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'compile'\u001b[0m\u001b[39m, <function Module.compile at \u001b[0m\u001b[1;36m0x1463e4280\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'compute_transition_scores'\u001b[0m\u001b[39m, <function GenerationMixin.compute_transition_scores at \u001b[0m\u001b[1;36m0x29b3b49d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'constrained_beam_search'\u001b[0m\u001b[39m, <function GenerationMixin.constrained_beam_search at \u001b[0m\u001b[1;36m0x29b3b21f0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'contrastive_search'\u001b[0m\u001b[39m, <function GenerationMixin.contrastive_search at \u001b[0m\u001b[1;36m0x29b3b4dc0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'cpu'\u001b[0m\u001b[39m, <function Module.cpu at \u001b[0m\u001b[1;36m0x1463dd550\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'create_extended_attention_mask_for_decoder'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function ModuleUtilsMixin.create_extended_attention_mask_for_decoder at \u001b[0m\u001b[1;36m0x29b3ed040\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'cuda'\u001b[0m\u001b[39m, <function Module.cuda at \u001b[0m\u001b[1;36m0x29b3efaf0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'deparallelize'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.deparallelize at \u001b[0m\u001b[1;36m0x29bce4820\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'disable_adapters'\u001b[0m\u001b[39m, <function PeftAdapterMixin.disable_adapters at \u001b[0m\u001b[1;36m0x29b3b2b80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'disable_input_require_grads'\u001b[0m\u001b[39m, <function PreTrainedModel.disable_input_require_grads at \u001b[0m\u001b[1;36m0x29b3edc10\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'double'\u001b[0m\u001b[39m, <function Module.double at \u001b[0m\u001b[1;36m0x1463dd700\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'enable_adapters'\u001b[0m\u001b[39m, <function PeftAdapterMixin.enable_adapters at \u001b[0m\u001b[1;36m0x29b3b2c10\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'enable_input_require_grads'\u001b[0m\u001b[39m, <function PreTrainedModel.enable_input_require_grads at \u001b[0m\u001b[1;36m0x29b3edb80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'estimate_tokens'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.estimate_tokens at \u001b[0m\u001b[1;36m0x29b3ed310\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'eval'\u001b[0m\u001b[39m, <function Module.eval at \u001b[0m\u001b[1;36m0x1463e0d30\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'extra_repr'\u001b[0m\u001b[39m, <function Module.extra_repr at \u001b[0m\u001b[1;36m0x1463e4040\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'float'\u001b[0m\u001b[39m, <function PreTrainedModel.float at \u001b[0m\u001b[1;36m0x29b3efca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'floating_point_ops'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.floating_point_ops at \u001b[0m\u001b[1;36m0x29b3ed3a0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'forward'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.forward at \u001b[0m\u001b[1;36m0x29bce4af0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'from_pretrained'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel.from_pretrained of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'generate'\u001b[0m\u001b[39m, <function GenerationMixin.generate at \u001b[0m\u001b[1;36m0x29b3b4ca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_adapter_state_dict'\u001b[0m\u001b[39m, <function PeftAdapterMixin.get_adapter_state_dict at \u001b[0m\u001b[1;36m0x29b3b2dc0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_buffer'\u001b[0m\u001b[39m, <function Module.get_buffer at \u001b[0m\u001b[1;36m0x1463dd0d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_extended_attention_mask'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.get_extended_attention_mask at \u001b[0m\u001b[1;36m0x29b3ed0d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_extra_state'\u001b[0m\u001b[39m, <function Module.get_extra_state at \u001b[0m\u001b[1;36m0x1463dd160\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_head_mask'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.get_head_mask at \u001b[0m\u001b[1;36m0x29b3ed160\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_input_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel.get_input_embeddings at \u001b[0m\u001b[1;36m0x29b3edca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_memory_footprint'\u001b[0m\u001b[39m, <function PreTrainedModel.get_memory_footprint at \u001b[0m\u001b[1;36m0x29b3efa60\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_output_embeddings'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.get_output_embeddings at \u001b[0m\u001b[1;36m0x29bce4670\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_parameter'\u001b[0m\u001b[39m, <function Module.get_parameter at \u001b[0m\u001b[1;36m0x1463dd040\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_position_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel.get_position_embeddings at \u001b[0m\u001b[1;36m0x29b3ef550\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'get_submodule'\u001b[0m\u001b[39m, <function Module.get_submodule at \u001b[0m\u001b[1;36m0x1463d6f70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'gradient_checkpointing_disable'\u001b[0m\u001b[39m, <function PreTrainedModel.gradient_checkpointing_disable at \u001b[0m\u001b[1;36m0x29b3ef820\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'gradient_checkpointing_enable'\u001b[0m\u001b[39m, <function PreTrainedModel.gradient_checkpointing_enable at \u001b[0m\u001b[1;36m0x29b3ef700\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'greedy_search'\u001b[0m\u001b[39m, <function GenerationMixin.greedy_search at \u001b[0m\u001b[1;36m0x29b3b4e50\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'group_beam_search'\u001b[0m\u001b[39m, <function GenerationMixin.group_beam_search at \u001b[0m\u001b[1;36m0x29b3b2160\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'half'\u001b[0m\u001b[39m, <function PreTrainedModel.half at \u001b[0m\u001b[1;36m0x29b3efc10\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'init_weights'\u001b[0m\u001b[39m, <function PreTrainedModel.init_weights at \u001b[0m\u001b[1;36m0x29b3ef5e0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'invert_attention_mask'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.invert_attention_mask at \u001b[0m\u001b[1;36m0x29b3ebf70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'ipu'\u001b[0m\u001b[39m, <function Module.ipu at \u001b[0m\u001b[1;36m0x1463dd430\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'load_adapter'\u001b[0m\u001b[39m, <function PeftAdapterMixin.load_adapter at \u001b[0m\u001b[1;36m0x29b3b29d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'load_state_dict'\u001b[0m\u001b[39m, <function Module.load_state_dict at \u001b[0m\u001b[1;36m0x1463e0700\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'load_tf_weights'\u001b[0m\u001b[39m, <function load_tf_weights_in_gpt2 at \u001b[0m\u001b[1;36m0x29bcb3f70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'modules'\u001b[0m\u001b[39m, <function Module.modules at \u001b[0m\u001b[1;36m0x1463e0b80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'named_buffers'\u001b[0m\u001b[39m, <function Module.named_buffers at \u001b[0m\u001b[1;36m0x1463e09d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'named_children'\u001b[0m\u001b[39m, <function Module.named_children at \u001b[0m\u001b[1;36m0x1463e0af0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'named_modules'\u001b[0m\u001b[39m, <function Module.named_modules at \u001b[0m\u001b[1;36m0x1463e0c10\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'named_parameters'\u001b[0m\u001b[39m, <function Module.named_parameters at \u001b[0m\u001b[1;36m0x1463e08b0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'num_parameters'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.num_parameters at \u001b[0m\u001b[1;36m0x29b3ed280\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'parallelize'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.parallelize at \u001b[0m\u001b[1;36m0x29bce4790\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'parameters'\u001b[0m\u001b[39m, <function Module.parameters at \u001b[0m\u001b[1;36m0x1463e0820\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'post_init'\u001b[0m\u001b[39m, <function PreTrainedModel.post_init at \u001b[0m\u001b[1;36m0x29b3ed5e0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'prepare_inputs_for_generation'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.prepare_inputs_for_generation at \u001b[0m\u001b[1;36m0x29bce4940\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'prune_heads'\u001b[0m\u001b[39m, <function PreTrainedModel.prune_heads at \u001b[0m\u001b[1;36m0x29b3ef670\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'push_to_hub'\u001b[0m\u001b[39m, <function PushToHubMixin.push_to_hub at \u001b[0m\u001b[1;36m0x29b3ebb80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_backward_hook'\u001b[0m\u001b[39m, <function Module.register_backward_hook at \u001b[0m\u001b[1;36m0x1463dda60\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_buffer'\u001b[0m\u001b[39m, <function Module.register_buffer at \u001b[0m\u001b[1;36m0x1463d6d30\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'register_for_auto_class'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<bound method PreTrainedModel.register_for_auto_class of <class \u001b[0m\u001b[32m'transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel'\u001b[0m\u001b[39m>>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_forward_hook'\u001b[0m\u001b[39m, <function Module.register_forward_hook at \u001b[0m\u001b[1;36m0x1463dddc0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_forward_pre_hook'\u001b[0m\u001b[39m, <function Module.register_forward_pre_hook at \u001b[0m\u001b[1;36m0x1463ddd30\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_full_backward_hook'\u001b[0m\u001b[39m, <function Module.register_full_backward_hook at \u001b[0m\u001b[1;36m0x1463ddaf0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_full_backward_pre_hook'\u001b[0m\u001b[39m, <function Module.register_full_backward_pre_hook at \u001b[0m\u001b[1;36m0x1463dd9d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_load_state_dict_post_hook'\u001b[0m\u001b[39m, <function Module.register_load_state_dict_post_hook at \u001b[0m\u001b[1;36m0x1463e05e0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_module'\u001b[0m\u001b[39m, <function Module.register_module at \u001b[0m\u001b[1;36m0x1463d6ee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_parameter'\u001b[0m\u001b[39m, <function Module.register_parameter at \u001b[0m\u001b[1;36m0x1463d6dc0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'register_state_dict_pre_hook'\u001b[0m\u001b[39m, <function Module.register_state_dict_pre_hook at \u001b[0m\u001b[1;36m0x1463e03a0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'requires_grad_'\u001b[0m\u001b[39m, <function Module.requires_grad_ at \u001b[0m\u001b[1;36m0x1463e0dc0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'reset_memory_hooks_state'\u001b[0m\u001b[39m, <function ModuleUtilsMixin.reset_memory_hooks_state at \u001b[0m\u001b[1;36m0x29b3ebdc0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'resize_position_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel.resize_position_embeddings at \u001b[0m\u001b[1;36m0x29b3ef4c0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'resize_token_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel.resize_token_embeddings at \u001b[0m\u001b[1;36m0x29b3ef1f0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'retrieve_modules_from_names'\u001b[0m\u001b[39m, <function PreTrainedModel.retrieve_modules_from_names at \u001b[0m\u001b[1;36m0x29b3efe50\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'reverse_bettertransformer'\u001b[0m\u001b[39m, <function PreTrainedModel.reverse_bettertransformer at \u001b[0m\u001b[1;36m0x29b3f20d0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'sample'\u001b[0m\u001b[39m, <function GenerationMixin.sample at \u001b[0m\u001b[1;36m0x29b3b4ee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'save_pretrained'\u001b[0m\u001b[39m, <function PreTrainedModel.save_pretrained at \u001b[0m\u001b[1;36m0x29b3ef940\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'set_adapter'\u001b[0m\u001b[39m, <function PeftAdapterMixin.set_adapter at \u001b[0m\u001b[1;36m0x29b3b2af0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'set_extra_state'\u001b[0m\u001b[39m, <function Module.set_extra_state at \u001b[0m\u001b[1;36m0x1463dd1f0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'set_input_embeddings'\u001b[0m\u001b[39m, <function PreTrainedModel.set_input_embeddings at \u001b[0m\u001b[1;36m0x29b3edd30\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'set_output_embeddings'\u001b[0m\u001b[39m, <function GPT2LMHeadModel.set_output_embeddings at \u001b[0m\u001b[1;36m0x29bce48b0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'share_memory'\u001b[0m\u001b[39m, <function Module.share_memory at \u001b[0m\u001b[1;36m0x1463e0ee0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'state_dict'\u001b[0m\u001b[39m, <function Module.state_dict at \u001b[0m\u001b[1;36m0x1463e04c0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'tie_weights'\u001b[0m\u001b[39m, <function PreTrainedModel.tie_weights at \u001b[0m\u001b[1;36m0x29b3edf70\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'to'\u001b[0m\u001b[39m, <function Module.to at \u001b[0m\u001b[1;36m0x29b3efb80\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'to_bettertransformer'\u001b[0m\u001b[39m, <function PreTrainedModel.to_bettertransformer at \u001b[0m\u001b[1;36m0x29b3f2040\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'to_empty'\u001b[0m\u001b[39m, <function Module.to_empty at \u001b[0m\u001b[1;36m0x1463dd8b0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'train'\u001b[0m\u001b[39m, <function Module.train at \u001b[0m\u001b[1;36m0x1463e0ca0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'type'\u001b[0m\u001b[39m, <function Module.type at \u001b[0m\u001b[1;36m0x1463dd5e0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[32m'warn_if_padding_and_no_attention_mask'\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   │   \u001b[0m\u001b[39m<function PreTrainedModel.warn_if_padding_and_no_attention_mask at \u001b[0m\u001b[1;36m0x29b3f2160\u001b[0m\u001b[39m>\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'xpu'\u001b[0m\u001b[39m, <function Module.xpu at \u001b[0m\u001b[1;36m0x1463dd4c0\u001b[0m\u001b[39m>\u001b[0m\u001b[1;39m)\u001b[0m\u001b[39m,\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[1;39m(\u001b[0m\u001b[32m'zero_grad'\u001b[0m\u001b[39m, <function Module.zero_grad at \u001b[0m\u001b[1;36m0x1463e0e50\u001b[0m\u001b[1m>\u001b[0m\u001b[1m)\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "predicate = inspect.isroutine\n",
    "GPT2LMHeadModel_methods_using_getmembers = list(get_members_of_function_or_method(GPT2LMHeadModel, predicate=predicate))\n",
    "\n",
    "pprint(sorted(GPT2LMHeadModel_methods_using_getmembers))"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "You can get the method resolution order (MRO) of a class via `cls.__mro__`. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "(transformers.models.gpt2.modeling_gpt2.GPT2LMHeadModel,\n",
       " transformers.models.gpt2.modeling_gpt2.GPT2PreTrainedModel,\n",
       " transformers.modeling_utils.PreTrainedModel,\n",
       " torch.nn.modules.module.Module,\n",
       " transformers.modeling_utils.ModuleUtilsMixin,\n",
       " transformers.generation.utils.GenerationMixin,\n",
       " transformers.utils.hub.PushToHubMixin,\n",
       " transformers.integrations.peft.PeftAdapterMixin,\n",
       " object)"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "inspect.getmro(GPT2LMHeadModel)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "A pseudocode to get all signatures of a class via MRO is as follows:\n",
    "\n",
    "```python\n",
    "def get_all_args(cls: Type[object]) -> Dict[str, inspect.Parameter]:\n",
    "    mro = inspect.getmro(cls)\n",
    "    all_args = {}\n",
    "    for base_class in mro[::-1]:  # reverse to start from topmost class\n",
    "        if base_class is object:  # skip the base 'object' class\n",
    "            continue\n",
    "        sig = inspect.signature(base_class.__init__)\n",
    "        all_args.update(sig.parameters)\n",
    "    return all_args\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Get Class and Instance Attributes"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__module__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__doc__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_protected_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_ChildClass__private_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'read_only_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'static_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__module__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__doc__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_protected_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_ChildClass__private_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__init__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'read_only_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'static_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__str__'\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span><span style=\"color: #008000; text-decoration-color: #008000\">'parent_instance_attr'</span>, <span style=\"color: #008000; text-decoration-color: #008000\">'instance_attr'</span>, <span style=\"color: #008000; text-decoration-color: #008000\">'instance_not_in_constructor_attr'</span>, <span style=\"color: #008000; text-decoration-color: #008000\">'_private_instance_attr'</span><span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\u001b[32m'parent_instance_attr'\u001b[0m, \u001b[32m'instance_attr'\u001b[0m, \u001b[32m'instance_not_in_constructor_attr'\u001b[0m, \u001b[32m'_private_instance_attr'\u001b[0m\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "pprint(list(class_child.__dict__.keys())) # class attributes\n",
    "pprint(list(instance_child.__dict__.keys())) # instance attributes"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<pre style=\"white-space:pre;overflow-x:auto;line-height:normal;font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace\"><span style=\"font-weight: bold\">[</span>\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__init__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_private_instance_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'read_only_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_not_in_constructor_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__str__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'parent_instance_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__doc__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'class_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_ChildClass__private_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'static_method'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'_protected_attr'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'__module__'</span>,\n",
       "<span style=\"color: #7fbf7f; text-decoration-color: #7fbf7f\">│   </span><span style=\"color: #008000; text-decoration-color: #008000\">'instance_method'</span>\n",
       "<span style=\"font-weight: bold\">]</span>\n",
       "</pre>\n"
      ],
      "text/plain": [
       "\u001b[1m[\u001b[0m\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__init__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_private_instance_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'read_only_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_not_in_constructor_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__str__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'parent_instance_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__doc__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'class_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_ChildClass__private_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'static_method'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'_protected_attr'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'__module__'\u001b[0m,\n",
       "\u001b[2;32m│   \u001b[0m\u001b[32m'instance_method'\u001b[0m\n",
       "\u001b[1m]\u001b[0m\n"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "union_class_and_instance_attributes = list(set(class_child.__dict__.keys()).union(set(instance_child.__dict__.keys())))\n",
    "pprint(union_class_and_instance_attributes)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Get Signature and Type Annotations of a Function"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "func_sig: Signature = inspect.signature(func)\n",
    "pprint(func_sig.parameters)\n",
    "pprint(func_sig.return_annotation)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Here are the 4 key properties of the `Parameter` object\n",
    "of the `Signature` object.\n",
    "\n",
    "```python\n",
    "@property\n",
    "def name(self):\n",
    "    return self._name\n",
    "\n",
    "@property\n",
    "def default(self):\n",
    "    return self._default\n",
    "\n",
    "@property\n",
    "def annotation(self):\n",
    "    return self._annotation\n",
    "\n",
    "@property\n",
    "def kind(self):\n",
    "    return self._kind\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "We will also use `get_type_hints` to get the type hints of a function\n",
    "instead of using the `annotations` property of `inspect.Signature`. The reason\n",
    "can be found in the docstring of `get_type_hints`:\n",
    "\n",
    "```python\n",
    "def get_type_hints(obj, globalns=None, localns=None, include_extras=False):\n",
    "    \"\"\"Return type hints for an object.\n",
    "\n",
    "    This is often the same as obj.__annotations__, but it handles\n",
    "    forward references encoded as string literals, adds Optional[t] if a\n",
    "    default value equal to None is set and recursively replaces all\n",
    "    'Annotated[T, ...]' with 'T' (unless 'include_extras=True').\n",
    "\n",
    "    ...\n",
    "    \"\"\"\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def no_type_hints(a, b, c, d, e, **kwargs):\n",
    "    return a, b, c, d, e, kwargs"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "get_type_hints(no_type_hints), inspect.signature(no_type_hints)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "How to know if a parameter is optional or not? We can use the `inspect.Parameter.empty`\n",
    "property."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "for name, value in inspect.signature(func).parameters.items():\n",
    "    print(value.default)\n",
    "    print(value.default is inspect.Parameter.empty)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "We will also use `__mro__` to get the method resolution order of a class\n",
    "because `__bases__` only returns the immediate parent class."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "ChildClass.__bases__, GPT2LMHeadModel.__bases__[0].__bases__[0].__bases__"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "list(reversed(inspect.getmro(GPT2LMHeadModel)))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def get_base_classes(cls: Type[Any], include_self: bool = False) -> Set[Type[Any]]:\n",
    "    \"\"\"\n",
    "    Get the base classes of a class and all its base classes.\n",
    "    \"\"\"\n",
    "    return set(cls.__mro__[0:-1] if include_self else cls.__mro__[1:-1])\n",
    "\n",
    "pprint(get_base_classes(GPT2LMHeadModel, include_self=True))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def get_default(param: Parameter) -> Any:\n",
    "    \"\"\"Return the parameter's default value or None if not specified.\"\"\"\n",
    "    return param.default if param.default is not param.empty else None\n",
    "\n",
    "def get_field_annotations(func_or_method: Callable[..., Any]) -> Tuple[List[Tuple[str, Any, Any]], Dict[str, Any]]:\n",
    "    if not inspect.isroutine(func_or_method):\n",
    "        raise ValueError(\"Expected a function or method\")\n",
    "\n",
    "    required_fields = []\n",
    "    optional_fields = []\n",
    "    annotations = {}\n",
    "\n",
    "    try:\n",
    "        sig: Signature = inspect.signature(func_or_method)\n",
    "        type_hints: Dict[str, Any] = get_type_hints(func_or_method)\n",
    "    except ValueError:\n",
    "        raise ValueError(\"Object does not support signature or type hints extraction.\") from None\n",
    "\n",
    "    for name, param in sig.parameters.items():\n",
    "        if name == \"self\":\n",
    "            continue\n",
    "\n",
    "        type_hint = type_hints.get(name, Any)\n",
    "        annotations[name] = type_hint\n",
    "        if param.default is param.empty:\n",
    "            required_fields.append((name, type_hint, Ellipsis))\n",
    "        else:\n",
    "            default_value = param.default\n",
    "            optional_fields.append((name, type_hint, default_value))\n",
    "\n",
    "    fields = required_fields + optional_fields\n",
    "    return fields, annotations\n",
    "\n",
    "\n",
    "# TODO: Tuple[str, Any, Any] should be Tuple[str, Any, ellipsis]\n",
    "def get_constructor_field_annotations(\n",
    "    cls: Type[Any], include_bases: bool = True\n",
    ") -> Tuple[List[Tuple[str, Any, Any]], Dict[str, Any]]:\n",
    "    fields = []\n",
    "    annotations = {}\n",
    "\n",
    "    classes_to_inspect = [cls] + list(get_base_classes(cls, include_self=False)) if include_bases else [cls]\n",
    "\n",
    "    for c in reversed(classes_to_inspect):  # Reverse to respect MRO\n",
    "        if hasattr(c, \"__init__\"):\n",
    "            class_fields, class_annotations = get_field_annotations(c.__init__)\n",
    "            # Update fields and annotations with those from the current class,\n",
    "            # avoiding duplicates.\n",
    "            for field in class_fields:\n",
    "                if field[0] not in annotations:\n",
    "                    fields.append(field)  # noqa: PERF401\n",
    "            annotations.update(class_annotations)\n",
    "\n",
    "    return fields, annotations"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "fields, annotations = get_constructor_field_annotations(TrainingArguments, include_bases=False)\n",
    "for field in fields:\n",
    "    assert len(field) == 3\n",
    "    print(f\"{field[0]}, {field[1]}, {field[2]}\")\n",
    "\n",
    "assert get_field_annotations(TrainingArguments.__init__) == (fields, annotations)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Warning: it does not play too well with `dataclass` and `pydantic` classes\n",
    "because they have more complex bells and whistles. However, because of the perks\n",
    "of `dataclass` and `pydantic`, we can just use\n",
    "[property](https://stackoverflow.com/questions/71183960/short-way-to-get-all-field-names-of-a-pydantic-class)\n",
    "like `model_fields` to get all fields and their types.\n",
    "\n",
    "As we can see from above, we did not handle `lr_scheduler_kwargs` well:\n",
    "\n",
    "```python\n",
    "lr_scheduler_kwargs, typing.Optional[typing.Dict], <factory>\n",
    "```\n",
    "\n",
    "where `<factory>` is the default value of the parameter. But it is actually\n",
    "referring to the `default_factory` of the `dataclass` field, which can be a default\n",
    "dict etc."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "def type_hint_to_str(type_hint: Any) -> str:\n",
    "    \"\"\"\n",
    "    Convert a type hint into its string representation.\n",
    "    \"\"\"\n",
    "    if hasattr(type_hint, '__name__'):\n",
    "        return type_hint.__name__\n",
    "    elif hasattr(type_hint, '_name') and type_hint._name is not None:\n",
    "        return str(type_hint._name)\n",
    "    elif type(type_hint) == _GenericAlias:  # For Python 3.8+\n",
    "        # Handles complex types, e.g., List[int], Union[str, int]\n",
    "        origin = type_hint_to_str(type_hint.__origin__)\n",
    "        args = ', '.join(type_hint_to_str(arg) for arg in type_hint.__args__)\n",
    "        return f\"{origin}[{args}]\"\n",
    "    else:\n",
    "        # Fallback for unhandled types\n",
    "        return str(type_hint)\n",
    "\n",
    "def create_config_class_str(fields: List[Tuple[str, Any, Any]]) -> str:\n",
    "    lines = [\"class Config:\"]\n",
    "    if not fields:\n",
    "        lines.append(\"    ...\")\n",
    "    else:\n",
    "        init_params = [\"self\"]\n",
    "        init_body = []\n",
    "        for name, type_hint, default in fields:\n",
    "            type_hint_str = type_hint_to_str(type_hint)\n",
    "            if default is Ellipsis:  # Required argument\n",
    "                param_str = f\"{name}: {type_hint_str}\"\n",
    "            elif default is field:\n",
    "                param_str = f\"{name}: {type_hint_str} = field(default_factory=dict)\"\n",
    "            else:\n",
    "                default_repr = repr(default) if default is not None else 'None'\n",
    "                param_str = f\"{name}: {type_hint_str} = {default_repr}\"\n",
    "\n",
    "            init_params.append(param_str)\n",
    "            init_body.append(f\"        self.{name} = {name}\")\n",
    "\n",
    "        lines.append(f\"    def __init__({', '.join(init_params)}):\")\n",
    "        lines.extend(init_body)\n",
    "\n",
    "    return '\\n'.join(lines)\n",
    "\n",
    "config_class_str = create_config_class_str(fields)\n",
    "print(config_class_str)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Using this as is will yield a `SyntaxError` because of the `<factory>` issue\n",
    "highlighted above. We can use on a \"normal\" class `Trainer`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "fields, annotations = get_constructor_field_annotations(Trainer, include_bases=False)\n",
    "config_class_str = create_config_class_str(fields)\n",
    "print(config_class_str)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import transformers\n",
    "import typing\n",
    "import torch\n",
    "from transformers import DataCollator\n",
    "\n",
    "NoneType = type(None)\n",
    "\n",
    "config_class_str = create_config_class_str(fields)\n",
    "\n",
    "# Execute the generated class definition string\n",
    "namespace = {}\n",
    "exec(config_class_str, globals(), namespace)\n",
    "\n",
    "# Extract the newly created class from the namespace\n",
    "ConfigClass = namespace['Config']\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "inspect.signature(ConfigClass.__init__)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## References and Further Readings\n",
    "\n",
    "- [inspect — Inspect live objects](https://docs.python.org/3/library/inspect.html)\n",
    "- [Getting attributes of a class](https://stackoverflow.com/questions/9058305/getting-attributes-of-a-class)\n",
    "- [Use __dict__ or vars()?](https://stackoverflow.com/questions/21297203/use-dict-or-vars)\n",
    "- [How do I get list of methods in a Python class?](https://stackoverflow.com/questions/1911281/how-do-i-get-list-of-methods-in-a-python-class)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "omniverse",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.18"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
